@@ -1,5 +1,4 @@
 #----------------------------------------------------------------------
-#----------------------------------------------------------------------
 # Name:        buildtools.config
 # Purpose:     Code to set and validate platform options and etc. for
 #              the wxPython build.  Moved to their own module and
@@ -8,7 +7,7 @@
 # Author:      Robin Dunn
 #
 # Created:     3-Nov-2010
-# Copyright:   (c) 2013-2018 by Total Control Software
+# Copyright:   (c) 2013-2020 by Total Control Software
 # License:     wxWindows License
 #----------------------------------------------------------------------
 
@@ -16,6 +15,7 @@ import sys
 import os
 import glob
 import fnmatch
+import shlex
 import re
 import shutil
 import subprocess
@@ -27,6 +27,8 @@ from distutils.dep_util  import newer
 
 import distutils.sysconfig
 
+from attrdict import AttrDict
+
 runSilently = False
 
 #----------------------------------------------------------------------
@@ -85,6 +87,9 @@ class Configuration(object):
     PKGDIR = 'wx'
     # The name of the top-level package
 
+    SIP_ABI = '12.8'
+    SIP_TRACE = False
+
     # ---------------------------------------------------------------
     # Basic initialization and configuration code
 
@@ -137,13 +142,16 @@ class Configuration(object):
         self.finishSetup()
 
 
-    def finishSetup(self, wx_config=None, debug=None):
+    def finishSetup(self, wx_config=None, debug=None, compiler=None):
         if wx_config is not None:
             self.WX_CONFIG = wx_config
 
         if debug is not None:
             self.debug = debug
 
+        if compiler is not None:
+            self.COMPILER = compiler
+
         #---------------------------------------
         # MSW specific settings
         if os.name == 'nt' and  self.COMPILER == 'msvc':
@@ -169,9 +177,10 @@ class Configuration(object):
                              ('WXUSINGDLL', '1'),
                              ('ISOLATION_AWARE_ENABLED', None),
                              #('NDEBUG',),  # using a 1-tuple makes it do an undef
-                             ('SIP_MODULE_NAME', 'wx.siplib'),
-                             ('SIP_MODULE_BASENAME', 'siplib'),
                              ]
+            if int(getVisCVersion()) > 100:
+                self.defines += [ ('wxUSE_RC_MANIFEST', '1'),
+                                  ('wxUSE_DPI_AWARE_MANIFEST', '2') ]
 
             self.libs = []
             self.libdirs = [ opj(self.WXDIR, 'lib', self.VCDLL) ]
@@ -193,6 +202,7 @@ class Configuration(object):
                             '/EHsc',
                             # '/GX-'  # workaround for internal compiler error in MSVC on some machines
                             ]
+            self.cxxflags = self.cflags[:]
             self.lflags = None
 
             # These confuse WAF, but since it already reliably picks the correct
@@ -204,6 +214,7 @@ class Configuration(object):
             # Other MSVC flags...
             # Uncomment these to have debug info for all kinds of builds
             #self.cflags += ['/Od', '/Z7']
+            #self.cxxflags += ['/Od', '/Z7']
             #self.lflags = ['/DEBUG', ]
 
 
@@ -217,13 +228,15 @@ class Configuration(object):
             self.libdirs = []
             self.libs = []
 
-            self.cflags = self.getWxConfigValue('--cxxflags')
-            self.cflags = self.cflags.split()
+            self.cflags = self.getWxConfigValue('--cflags').split()
+            self.cxxflags = self.getWxConfigValue('--cxxflags').split()
             if self.debug:
-                self.cflags.append('-ggdb')
-                self.cflags.append('-O0')
+                for lst in [self.cflags, self.cxxflags]:
+                    lst.append('-ggdb')
+                    lst.append('-O0')
             else:
-                self.cflags.append('-O3')
+                for lst in [self.cflags, self.cxxflags]:
+                    lst.append('-O3')
 
             lflags = self.getWxConfigValue('--libs')
             self.MONOLITHIC = (lflags.find("_xrc") == -1)
@@ -233,9 +246,47 @@ class Configuration(object):
             self.WXRELEASE  = self.getWxConfigValue('--release')
             self.WXPREFIX   = self.getWxConfigValue('--prefix')
 
+            # Set CC, CXX and maybe LDSHARED based on what was configured for
+            # wxWidgets, but not if those values are in the environment.
             self.CC = self.CXX = self.LDSHARED = None
+            if not os.environ.get('CC'):
+                compiler, flags = self.unpackCompilerCommand(self.getWxConfigValue('--cc'))
+                self.CC = os.environ["CC"] = compiler
+                for flag in reversed(flags):
+                    if flag not in self.cflags:
+                        self.cflags.insert(0, flag)
 
-            # wxMac settings
+            if not os.environ.get('CXX'):
+                compiler, flags = self.unpackCompilerCommand(self.getWxConfigValue('--cxx'))
+                self.CXX = os.environ["CXX"] = compiler
+                for flag in reversed(flags):
+                    if flag not in self.cxxflags:
+                        self.cxxflags.insert(0, flag)
+
+            if sys.platform[:6] == "darwin" and not os.environ.get('LDSHARED'):
+                # We want to use the linker command from wx to make sure
+                # we get the right sysroot, but we also need to ensure that
+                # the other linker flags that distutils wants to use are
+                # included as well.
+                LDSHARED = distutils.sysconfig.get_config_var('LDSHARED').split()
+                # remove the compiler command
+                del LDSHARED[0]
+                # remove any -sysroot flags and their arg
+                while True:
+                    try:
+                        index = LDSHARED.index('-isysroot')
+                        # Strip this argument and the next one:
+                        del LDSHARED[index:index+2]
+                    except ValueError:
+                        break
+                LDSHARED = ' '.join(LDSHARED)
+                # Combine with wx's ld command and stash it in the env
+                # where distutils will get it later.
+                LDSHARED = self.getWxConfigValue('--ld').replace(' -o', '') + ' ' + LDSHARED
+                self.LDSHARED = os.environ["LDSHARED"]  = LDSHARED
+
+
+            # Other wxMac-only settings
             if sys.platform[:6] == "darwin":
                 self.WXPLAT = '__WXMAC__'
 
@@ -245,41 +296,17 @@ class Configuration(object):
                 else:
                     self.WXPLAT2 = '__WXOSX_COCOA__'
 
-                self.libs = ['stdc++']
                 if not self.ARCH == "":
+
                     for arch in self.ARCH.split(','):
-                        self.cflags.append("-arch")
-                        self.cflags.append(arch)
+                        for lst in [self.cflags, self.cxxflags]:
+                            lst.append("-arch")
+                            lst.append(arch)
                         self.lflags.append("-arch")
                         self.lflags.append(arch)
 
-                if not os.environ.get('CC') or not os.environ.get('CXX'):
-                    self.CC =  os.environ["CC"]  = self.getWxConfigValue('--cc')
-                    self.CXX = os.environ["CXX"] = self.getWxConfigValue('--cxx')
 
-                    # We want to use the linker command from wx to make sure
-                    # we get the right sysroot, but we also need to ensure that
-                    # the other linker flags that distutils wants to use are
-                    # included as well.
-                    LDSHARED = distutils.sysconfig.get_config_var('LDSHARED').split()
-                    # remove the compiler command
-                    del LDSHARED[0]
-                    # remove any -sysroot flags and their arg
-                    while 1:
-                        try:
-                            index = LDSHARED.index('-isysroot')
-                            # Strip this argument and the next one:
-                            del LDSHARED[index:index+2]
-                        except ValueError:
-                            break
-                    LDSHARED = ' '.join(LDSHARED)
-                    # Combine with wx's ld command and stash it in the env
-                    # where distutils will get it later.
-                    LDSHARED = self.getWxConfigValue('--ld').replace(' -o', '') + ' ' + LDSHARED
-                    self.LDSHARED = os.environ["LDSHARED"]  = LDSHARED
-
-
-            # wxGTK settings
+            # wxGTK-only settings
             else:
                 # Set flags for other Unix type platforms
                 if self.WXPORT == 'gtk':
@@ -294,7 +321,7 @@ class Configuration(object):
                     self.WXPLAT = '__WXGTK__'
                     portcfg = os.popen('pkg-config gtk+-3.0 --cflags', 'r').read()[:-1]
                 elif self.WXPORT == 'x11':
-                    msg("WARNING: The wxX11 port is no supported")
+                    msg("WARNING: The wxX11 port is not supported")
                     self.WXPLAT = '__WXX11__'
                     portcfg = ''
                     self.BUILD_BASE = self.BUILD_BASE + '-' + self.WXPORT
@@ -305,6 +332,7 @@ class Configuration(object):
                     raise SystemExit("Unknown WXPORT value: " + self.WXPORT)
 
                 self.cflags += portcfg.split()
+                self.cxxflags += portcfg.split()
 
                 # Some distros (e.g. Mandrake) put libGLU in /usr/X11R6/lib, but
                 # wx-config doesn't output that for some reason.  For now, just
@@ -316,9 +344,11 @@ class Configuration(object):
             # Move the various -I, -D, etc. flags we got from the config scripts
             # into the distutils lists.
             self.cflags = self.adjustCFLAGS(self.cflags, self.defines, self.includes)
+            self.cxxflags = self.adjustCFLAGS(self.cxxflags, self.defines, self.includes)
             self.lflags = self.adjustLFLAGS(self.lflags, self.libdirs, self.libs)
 
             self.cflags.insert(0, '-UNDEBUG')
+            self.cxxflags.insert(0, '-UNDEBUG')
 
             if self.debug and self.WXPORT == 'msw' and self.COMPILER != 'mingw32':
                 self.defines.append( ('_DEBUG', None) )
@@ -347,10 +377,9 @@ class Configuration(object):
         # then the version number is built without a revision number. IOW, it
         # is a release build.  (In theory)
         if os.path.exists('REV.txt'):
-            f = open('REV.txt')
-            self.VER_FLAGS += f.read().strip()
+            with open('REV.txt') as f:
+                self.VER_FLAGS += f.read().strip()
             self.BUILD_TYPE = 'snapshot'
-            f.close()
         elif os.environ.get('WXPYTHON_RELEASE') == 'yes':
             self.BUILD_TYPE = 'release'
         else:
@@ -398,7 +427,7 @@ class Configuration(object):
         """
         # if WX_CONFIG hasn't been set to an explicit value then construct one.
         if self.WX_CONFIG is None:
-            self.WX_CONFIG='wx-config'
+            self.WX_CONFIG = os.environ.get('WX_CONFIG', 'wx-config')
             port = self.WXPORT
             if port == "x11":
                 port = "x11univ"
@@ -429,6 +458,16 @@ class Configuration(object):
         return value
 
 
+    def unpackCompilerCommand(self, cmd):
+        """
+        It's possible for the CC and CXX values coming from wx-config to have
+        some extra parameters tacked on. Let's split them apart.
+        """
+        cmd = shlex.split(cmd)
+        compiler = cmd[0]
+        flags = cmd[1:]
+        return compiler, flags
+
 
     def build_locale_dir(self, destdir, verbose=1):
         """Build a locale dir under the wxPython package."""
@@ -438,6 +477,7 @@ class Configuration(object):
             dest = opj(destdir, lang, 'LC_MESSAGES')
             mkpath(dest, verbose=verbose)
             copy_file(src, opj(dest, 'wxstd.mo'), update=1, verbose=verbose)
+            os.unlink(src)
             self.CLEANUP.append(opj(dest, 'wxstd.mo'))
             self.CLEANUP.append(dest)
 
@@ -673,10 +713,11 @@ def _getSbfValue(etg, keyName):
     cfg = Config()
     sbf = opj(cfg.SIPOUT, etg.NAME + '.sbf')
     out = list()
-    for line in open(sbf):
-        key, value = line.split('=')
-        if key.strip() == keyName:
-            return sorted([opj(cfg.SIPOUT, v) for v in value.strip().split()])
+    with open(sbf) as fid:
+        for line in fid:
+            key, value = line.split('=')
+            if key.strip() == keyName:
+                return sorted([opj(cfg.SIPOUT, v) for v in value.strip().split()])
     return None
 
 def getEtgSipCppFiles(etg):
@@ -745,16 +786,14 @@ def writeIfChanged(filename, text):
     """
 
     if os.path.exists(filename):
-        f = textfile_open(filename, 'rt')
-        current = f.read()
-        f.close()
+        with textfile_open(filename, 'rt') as f:
+            current = f.read()
 
         if current == text:
             return
 
-    f = textfile_open(filename, 'wt')
-    f.write(text)
-    f.close()
+    with textfile_open(filename, 'wt') as f:
+        f.write(text)
 
 
 # TODO: we might be able to get rid of this when the install code is updated...
@@ -904,7 +943,7 @@ def textfile_open(filename, mode='rt'):
 
 def getSipFiles(names):
     """
-    Returns a list of the coresponding .sip files for each of the names in names.
+    Returns a list of the corresponding .sip files for each of the names in names.
     """
     files = list()
     for template in ['sip/gen/%s.sip', 'src/%s.sip']:
@@ -917,20 +956,71 @@ def getSipFiles(names):
 
 
 def getVisCVersion():
-    text = runcmd("cl.exe", getOutput=True, echoCmd=False)
-    if 'Version 13' in text:
-        return '71'
-    if 'Version 15' in text:
-        return '90'
-    if 'Version 16' in text:
-        return '100'
-    if 'Version 18' in text:
-        return '120'
-    if 'Version 19' in text:
-        return '140'
-    # TODO: Add more tests to get the other versions...
+    if MSVCinfo is None:
+        raise RuntimeError('getMSVCInfo has not been called yet.')
+    # Convert a float like 14.28 to 140, for historical reasons
+    # TODO: decide on switching to 142, 143, etc.??
+    ver = str(int(MSVCinfo.vc_ver)) + '0'
+    return ver
+
+
+def getExpectedVisCVersion():
+    """
+    Returns the Visual C version that Python is expecting, based on the usual
+    version that stock Python was built with.
+    (Not currently used, we're just selecting the latest available compiler
+    >= 14.0 for now...)
+    """
+    if MSVCinfo is None:
+        raise RuntimeError('getMSVCInfo has not been called yet.')
+    py_ver = MSVCinfo.py_ver
+    if py_ver in ((3, 5), (3, 6), (3, 7), (3, 8)):
+        min_ver = 14.0
+    elif py_ver in ((3, 9), (3, 10)):
+        min_ver = 14.2
     else:
-        return 'FIXME'
+        raise RuntimeError('This library does not support python version %d.%d' % py_version)
+    return min_ver
+
+
+MSVCinfo = None
+def getMSVCInfo(PYTHON, arch, set_env=False):
+    """
+    Fetch info from the system about MSVC, such as versions, paths, etc.
+    """
+    global MSVCinfo
+    if MSVCinfo is not None:
+        return MSVCinfo
+
+    # Note that it starts with a monkey-patch in setuptools.msvc to
+    # workaround this issue: pypa/setuptools#1902
+    cmd = \
+        "import os, sys, setuptools.msvc; " \
+        "setuptools.msvc.isfile = lambda path: path is not None and os.path.isfile(path); " \
+        "ei = setuptools.msvc.EnvironmentInfo('{}', vc_min_ver=14.0); " \
+        "env = ei.return_env(); " \
+        "env['vc_ver'] = ei.vc_ver; " \
+        "env['vs_ver'] = ei.vs_ver; " \
+        "env['arch'] = ei.pi.arch; " \
+        "env['py_ver'] = sys.version_info[:2]; " \
+        "print(env)"
+    cmd = cmd.format(arch)
+    env = eval(runcmd('"%s" -c "%s"' % (PYTHON, cmd), getOutput=True, echoCmd=False))
+    info = AttrDict(env)
+
+    if set_env:
+        os.environ['PATH'] =    info.path
+        os.environ['INCLUDE'] = info.include
+        os.environ['LIB'] =     info.lib
+        os.environ['LIBPATH'] = info.libpath
+
+        # We already have everything we need, tell distutils to not go hunting
+        # for it all again if it happens to be called.
+        os.environ['DISTUTILS_USE_SDK'] = "1"
+        os.environ['MSSdk'] = "1"
+
+    MSVCinfo = info
+    return info
 
 
 _haveObjDump = None
