@@ -6,7 +6,7 @@
 # Author:       Robin Dunn
 #
 # Created:      A long time ago, in a galaxy far, far away...
-# Copyright:    (c) 1999-2018 by Total Control Software
+# Copyright:    (c) 1999-2020 by Total Control Software
 # Licence:      wxWindows license
 # Tags:         phoenix-port, py3-port
 #----------------------------------------------------------------------------
@@ -171,8 +171,7 @@ def RemoveHTMLTags(data):
 
 def FormatDocs(keyword, values, num):
 
-    names = list(values.keys())
-    names.sort()
+    names = sorted(values)
 
     headers = (num == 2 and [_eventHeaders] or [_styleHeaders])[0]
     table = (num == 2 and [_eventTable] or [_styleTable])[0]
@@ -347,8 +346,8 @@ def FindImages(text, widgetName):
         if "src=" in items:
             possibleImage = items.replace("src=", "").strip()
             possibleImage = possibleImage.replace('"', "")
-            f = urllib.request.urlopen(_trunkURL + possibleImage)
-            stream = f.read()
+            with urllib.request.urlopen(_trunkURL + possibleImage) as f:
+                stream = f.read()
         elif "alt=" in items:
             plat = items.replace("alt=", "").replace("'", "").strip()
             path = os.path.join(imagesDir, plat, widgetName + ".png")
@@ -385,7 +384,7 @@ def GetCaretPeriod(win = None):
 
         # attempt to average.
         # (wx systemsettings allows on and off time, but scintilla just takes a single period.)
-        return (onmsec + offmsec) / 2.0
+        return int((onmsec + offmsec) / 2.0)
 
     except AttributeError:
         # Issue where wx.SYS_CARET_ON/OFF_MSEC is unavailable.
@@ -418,12 +417,12 @@ class InternetThread(Thread):
 
         try:
             url = _docsURL % ReplaceCapitals(self.selectedClass)
-            fid = urllib.request.urlopen(url)
+            with urllib.request.urlopen(url) as fid:
+                if six.PY2:
+                    originalText = fid.read()
+                else:
+                    originalText = fid.read().decode("utf-8")
 
-            if six.PY2:
-                originalText = fid.read()
-            else:
-                originalText = fid.read().decode("utf-8")
             text = RemoveHTMLTags(originalText).split("\n")
             data = FindWindowStyles(text, originalText, self.selectedClass)
 
@@ -527,7 +526,7 @@ try:
         def SetUpEditor(self):
             """
             This method carries out the work of setting up the demo editor.
-            It's seperate so as not to clutter up the init code.
+            It's separate so as not to clutter up the init code.
             """
             import keyword
 
@@ -833,12 +832,9 @@ class DemoCodePanel(wx.Panel):
                 wx.LogMessage("Created directory for modified demos: %s" % GetModifiedDirectory())
 
         # Save
-        f = open(modifiedFilename, "wt")
         source = self.editor.GetText()
-        try:
+        with open(modifiedFilename, "wt") as f:
             f.write(source)
-        finally:
-            f.close()
 
         busy = wx.BusyInfo("Reloading demo module...")
         self.demoModules.LoadFromFile(modModified, modifiedFilename)
@@ -964,9 +960,8 @@ def GetDocImagesDir():
 
 def SearchDemo(name, keyword):
     """ Returns whether a demo contains the search keyword or not. """
-    fid = open(GetOriginalFilename(name), "rt")
-    fullText = fid.read()
-    fid.close()
+    with open(GetOriginalFilename(name), "rt") as fid:
+        fullText = fid.read()
 
     if six.PY2:
         fullText = fullText.decode("iso-8859-1")
@@ -1014,10 +1009,7 @@ def HuntExternalDemos():
 
     # Sort and reverse the external demos keys so that they
     # come back in alphabetical order
-    keys = list(externalDemos.keys())
-    keys.sort()
-    keys.reverse()
-
+    keys = sorted(externalDemos, reverse=True)
     # Loop over all external packages
     for extern in keys:
         package = externalDemos[extern]
@@ -1100,16 +1092,13 @@ class DemoModules(object):
 
     def LoadFromFile(self, modID, filename):
         self.modules[modID][2] = filename
-        file = open(filename, "rt")
-        self.LoadFromSource(modID, file.read())
-        file.close()
-
+        with open(filename, "rt") as file_:
+            self.LoadFromSource(modID, file_.read())
 
     def LoadFromSource(self, modID, source):
         self.modules[modID][1] = source
         self.LoadDict(modID)
 
-
     def LoadDict(self, modID):
         if self.name != __name__:
             source = self.modules[modID][1]
@@ -1176,13 +1165,8 @@ class DemoModules(object):
 
         source = self.modules[modID][1]
         filename = self.modules[modID][2]
-
-        try:
-            file = open(filename, "wt")
-            file.write(source)
-        finally:
-            file.close()
-
+        with open(filename, "wt") as file_:
+            file_.write(source)
 
     def Delete(self, modID):
         if self.modActive == modID:
@@ -1280,7 +1264,7 @@ class DemoErrorPanel(wx.Panel):
         self.list.SetColumnWidth(2, wx.LIST_AUTOSIZE)
         self.box.Add(wx.StaticText(self, -1, "Traceback:")
                      , 0, wx.ALIGN_CENTER | wx.TOP, 5)
-        self.box.Add(self.list, 1, wx.GROW | wx.ALIGN_CENTER | wx.ALL, 5)
+        self.box.Add(self.list, 1, wx.EXPAND | wx.ALL, 5)
         self.box.Add(wx.StaticText(self, -1, "Entries from the demo module are shown in blue\n"
                                            + "Double-click on them to go to the offending line")
                      , 0, wx.ALIGN_CENTER | wx.BOTTOM, 5)
@@ -1390,7 +1374,7 @@ class DemoTaskBarIcon(TaskBarIcon):
             img = img.Scale(16, 16)
         elif "wxGTK" in wx.PlatformInfo:
             img = img.Scale(22, 22)
-        # wxMac can be any size upto 128x128, so leave the source img alone....
+        # wxMac can be any size up to 128x128, so leave the source img alone....
         icon = wx.Icon(img.ConvertToBitmap())
         return icon
 
@@ -1501,15 +1485,14 @@ class wxPythonDemo(wx.Frame):
 
         # Create a Notebook
         self.nb = wx.Notebook(pnl, -1, style=wx.CLIP_CHILDREN)
-        if 'wxMac' not in wx.PlatformInfo:
-            imgList = wx.ImageList(16, 16)
-            for png in ["overview", "code", "demo"]:
-                bmp = images.catalog[png].GetBitmap()
-                imgList.Add(bmp)
-            for indx in range(9):
-                bmp = images.catalog["spinning_nb%d"%indx].GetBitmap()
-                imgList.Add(bmp)
-            self.nb.AssignImageList(imgList)
+        imgList = wx.ImageList(16, 16)
+        for png in ["overview", "code", "demo"]:
+            bmp = images.catalog[png].GetBitmap()
+            imgList.Add(bmp)
+        for indx in range(9):
+            bmp = images.catalog["spinning_nb%d"%indx].GetBitmap()
+            imgList.Add(bmp)
+        self.nb.AssignImageList(imgList)
 
         self.BuildMenuBar()
 
@@ -1553,23 +1536,9 @@ class wxPythonDemo(wx.Frame):
         self.tree.Bind(wx.EVT_LEFT_DOWN, self.OnTreeLeftDown)
 
         # Set up a wx.html.HtmlWindow on the Overview Notebook page
-        # we put it in a panel first because there seems to be a
-        # refresh bug of some sort (wxGTK) when it is directly in
-        # the notebook...
-
-        if 0:  # the old way
-            self.ovr = wx.html.HtmlWindow(self.nb, -1, size=(400, 400))
-            self.nb.AddPage(self.ovr, self.overviewText, imageId=0)
-
-        else:  # hopefully I can remove this hacky code soon, see SF bug #216861
-            panel = wx.Panel(self.nb, -1, style=wx.CLIP_CHILDREN)
-            self.ovr = wx.html.HtmlWindow(panel, -1, size=(400, 400))
-            self.nb.AddPage(panel, self.overviewText, imageId=0)
-
-            def OnOvrSize(evt, ovr=self.ovr):
-                ovr.SetSize(evt.GetSize())
-            panel.Bind(wx.EVT_SIZE, OnOvrSize)
-            panel.Bind(wx.EVT_ERASE_BACKGROUND, EmptyHandler)
+        # for displaying info about each sample in the demo.
+        self.ovr = wx.html.HtmlWindow(self.nb, -1, size=(400, 400))
+        self.nb.AddPage(self.ovr, self.overviewText, imageId=0)
 
         if "gtk2" in wx.PlatformInfo or "gtk3" in wx.PlatformInfo:
             self.ovr.SetStandardFonts()
@@ -1676,14 +1645,11 @@ class wxPythonDemo(wx.Frame):
             self.pickledData = {}
             return
 
-        fid = open(pickledFile, "rb")
-        try:
-            self.pickledData = cPickle.load(fid)
-        except:
-            self.pickledData = {}
-
-        fid.close()
-
+        with open(pickledFile, "rb") as fid:
+            try:
+                self.pickledData = cPickle.load(fid)
+            except:
+                self.pickledData = {}
 
     def BuildMenuBar(self):
 
@@ -1695,8 +1661,7 @@ class wxPythonDemo(wx.Frame):
                            wx.ITEM_CHECK)
         self.Bind(wx.EVT_MENU, self.OnToggleRedirect, item)
 
-        wx.App.SetMacExitMenuItemId(9123)
-        exitItem = wx.MenuItem(menu, 9123, 'E&xit\tCtrl-Q', 'Get the heck outta here!')
+        exitItem = wx.MenuItem(menu, wx.ID_EXIT, 'E&xit\tCtrl-Q', 'Get the heck outta here!')
         exitItem.SetBitmap(images.catalog['exit'].GetBitmap())
         menu.Append(exitItem)
         self.Bind(wx.EVT_MENU, self.OnFileExit, exitItem)
@@ -1734,8 +1699,7 @@ class wxPythonDemo(wx.Frame):
         item.Check(self.allowAuiFloating)
         self.Bind(wx.EVT_MENU, self.OnAllowAuiFloating, item)
 
-        auiPerspectives = list(self.auiConfigurations.keys())
-        auiPerspectives.sort()
+        auiPerspectives = sorted(self.auiConfigurations)
         perspectivesMenu = wx.Menu()
         item = wx.MenuItem(perspectivesMenu, -1, DEFAULT_PERSPECTIVE, "Load startup default perspective", wx.ITEM_RADIO)
         self.Bind(wx.EVT_MENU, self.OnAUIPerspectives, item)
@@ -1747,7 +1711,7 @@ class wxPythonDemo(wx.Frame):
             perspectivesMenu.Append(item)
             self.Bind(wx.EVT_MENU, self.OnAUIPerspectives, item)
 
-        menu.Append(wx.ID_ANY, "&AUI Perspectives", perspectivesMenu)
+        menu.AppendSubMenu(perspectivesMenu, "&AUI Perspectives")
         self.perspectives_menu = perspectivesMenu
 
         item = wx.MenuItem(menu, -1, 'Save Perspective', 'Save AUI perspective')
@@ -2279,7 +2243,7 @@ class wxPythonDemo(wx.Frame):
 
     def OnToggleRedirect(self, event):
         app = wx.GetApp()
-        if event.Checked():
+        if event.IsChecked():
             app.RedirectStdio()
             print("Print statements and other standard output will now be directed to this window.")
         else:
@@ -2325,7 +2289,7 @@ class wxPythonDemo(wx.Frame):
 
     def OnAllowAuiFloating(self, event):
 
-        self.allowAuiFloating = event.Checked()
+        self.allowAuiFloating = event.IsChecked()
         for pane in self.mgr.GetAllPanes():
             if pane.name != "Notebook":
                 pane.Floatable(self.allowAuiFloating)
@@ -2413,7 +2377,7 @@ class wxPythonDemo(wx.Frame):
         about.Destroy()
 
     def OnHelpFind(self, event):
-        if self.finddlg != None:
+        if self.finddlg is not None:
             return
 
         self.nb.SetSelection(1)
@@ -2423,7 +2387,7 @@ class wxPythonDemo(wx.Frame):
 
 
     def OnUpdateFindItems(self, evt):
-        evt.Enable(self.finddlg == None)
+        evt.Enable(self.finddlg is None)
 
 
     def OnFind(self, event):
@@ -2536,9 +2500,8 @@ class wxPythonDemo(wx.Frame):
 
         MakeDocDirs()
         pickledFile = GetDocFile()
-        fid = open(pickledFile, "wb")
-        cPickle.dump(self.pickledData, fid, cPickle.HIGHEST_PROTOCOL)
-        fid.close()
+        with open(pickledFile, "wb") as fid:
+            cPickle.dump(self.pickledData, fid, cPickle.HIGHEST_PROTOCOL)
 
         self.Destroy()
 
@@ -2594,7 +2557,7 @@ class wxPythonDemo(wx.Frame):
 
     #---------------------------------------------
     def OnIconfiy(self, evt):
-        wx.LogMessage("OnIconfiy: %s" % evt.Iconized())
+        wx.LogMessage("OnIconfiy: %s" % evt.IsIconized())
         evt.Skip()
 
     #---------------------------------------------
@@ -2622,7 +2585,7 @@ class MySplashScreen(SplashScreen):
                                  wx.adv.SPLASH_CENTRE_ON_SCREEN | wx.adv.SPLASH_TIMEOUT,
                                  5000, None, -1)
         self.Bind(wx.EVT_CLOSE, self.OnClose)
-        self.fc = wx.CallLater(2000, self.ShowMain)
+        self.fc = wx.CallLater(1000, self.ShowMain)
 
 
     def OnClose(self, evt):
@@ -2706,14 +2669,14 @@ class MyApp(wx.App, wx.lib.mixins.inspection.InspectionMixin):
 
         self.InitInspection()  # for the InspectionMixin base class
 
-        # Now that we've warned the user about possibile problems,
+        # Now that we've warned the user about possible problems,
         # lets import images
         import images as i
         global images
         images = i
 
         # For debugging
-        #self.SetAssertMode(wx.PYAPP_ASSERT_DIALOG|wx.PYAPP_ASSERT_EXCEPTION)
+        #self.SetAssertMode(wx.APP_ASSERT_DIALOG|wx.APP_ASSERT_EXCEPTION)
 
         wx.SystemOptions.SetOption("mac.window-plain-transition", 1)
         self.SetAppName("wxPyDemo")
@@ -2731,6 +2694,9 @@ class MyApp(wx.App, wx.lib.mixins.inspection.InspectionMixin):
 
         return True
 
+    def InitLocale(self):
+        super().InitLocale()
+        self._locale = wx.Locale(wx.LANGUAGE_ENGLISH)
 
 
 #---------------------------------------------------------------------------
