@@ -929,7 +929,7 @@ class HyperTreeHeaderRenderer(object):
 
     def DrawHeaderButton(self, dc, rect, flags=0, params=None):
 
-        if params != None:
+        if params is not None:
             text_align = params.m_labelAlignment
             bitmap     = params.m_labelBitmap
             text_color = params.m_labelColour
@@ -965,7 +965,7 @@ class HyperTreeHeaderRenderer(object):
 
         dc.SetBackgroundMode(wx.TRANSPARENT)
 
-        if params == None:
+        if params is None:
             return
 
         # We need to draw the text and/or icon bitmap
@@ -1055,7 +1055,7 @@ class HyperTreeListDemo(wx.Frame):
         splitter.SplitVertically(self.leftpanel, panel, 300)
 
         splitter.SetMinimumPaneSize(120)
-        self.leftpanel.SetBackgroundColour(wx.WHITE)
+        #self.leftpanel.SetBackgroundColour(wx.WHITE)
         self.leftpanel.SetScrollRate(20, 20)
 
         self.Fit()
@@ -1093,7 +1093,7 @@ class HyperTreeListDemo(wx.Frame):
         msg = "This Is The About Dialog Of The HyperTreeList Demo.\n\n" + \
               "Author: Andrea Gavana @ 08 May 2007\n\n" + \
               "Please Report Any Bug/Requests Of Improvements\n" + \
-              "To Me At The Following Adresses:\n\n" + \
+              "To Me At The Following AdresAddresseses:\n\n" + \
               "andrea.gavana@maerskoil.com\n" + "andrea.gavana@gmail.com\n\n" + \
               "Welcome To wxPython " + wx.VERSION_STRING + "!!"
 
@@ -1233,7 +1233,7 @@ class HyperTreeListDemo(wx.Frame):
         buttonconnection.Bind(wx.EVT_BUTTON, self.OnButtonConnection)
         sizer1.Add(label, 0, wx.ALL|wx.ALIGN_CENTER, 5)
         sizer1.Add((1,0), 1, wx.EXPAND)
-        sizer1.Add(buttonconnection, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_RIGHT, 5)
+        sizer1.Add(buttonconnection, 0, wx.ALL, 5)
 
         sizer2 = wx.BoxSizer(wx.HORIZONTAL)
         label = wx.StaticText(self.leftpanel, -1, "Border Pen")
@@ -1244,7 +1244,7 @@ class HyperTreeListDemo(wx.Frame):
         buttonborder.Bind(wx.EVT_BUTTON, self.OnButtonBorder)
         sizer2.Add(label, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.ALIGN_CENTER, 5)
         sizer2.Add((1,0), 1, wx.EXPAND)
-        sizer2.Add(buttonborder, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_RIGHT, 5)
+        sizer2.Add(buttonborder, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM, 5)
 
         sizer3 = wx.BoxSizer(wx.HORIZONTAL)
         label = wx.StaticText(self.leftpanel, -1, "Tree Buttons")
@@ -1255,7 +1255,7 @@ class HyperTreeListDemo(wx.Frame):
         buttontree.Bind(wx.EVT_BUTTON, self.OnButtonTree)
         sizer3.Add(label, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.ALIGN_CENTER, 5)
         sizer3.Add((1,0), 1, wx.EXPAND)
-        sizer3.Add(buttontree, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_RIGHT, 5)
+        sizer3.Add(buttontree, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM, 5)
 
         sizer4 = wx.BoxSizer(wx.HORIZONTAL)
         label = wx.StaticText(self.leftpanel, -1, "Check/Radio Buttons")
@@ -1266,7 +1266,7 @@ class HyperTreeListDemo(wx.Frame):
         buttoncr.Bind(wx.EVT_BUTTON, self.OnButtonCheckRadio)
         sizer4.Add(label, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.ALIGN_CENTER, 5)
         sizer4.Add((1,0), 1, wx.EXPAND)
-        sizer4.Add(buttoncr, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_RIGHT, 5)
+        sizer4.Add(buttoncr, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM, 5)
 
         sizer5 = wx.BoxSizer(wx.HORIZONTAL)
         radioimage = wx.RadioButton(self.leftpanel, -1, "Image Background", style=wx.RB_GROUP)
@@ -1275,7 +1275,7 @@ class HyperTreeListDemo(wx.Frame):
         self.imagebutton.Bind(wx.EVT_BUTTON, self.OnChooseImage)
         sizer5.Add(radioimage, 0, wx.ALL|wx.ALIGN_CENTER, 5)
         sizer5.Add((1,0), 1, wx.EXPAND)
-        sizer5.Add(self.imagebutton, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_RIGHT, 5)
+        sizer5.Add(self.imagebutton, 0, wx.ALL, 5)
 
         sizer6 = wx.BoxSizer(wx.HORIZONTAL)
         radiobackground = wx.RadioButton(self.leftpanel, -1, "Background Colour")
@@ -1307,7 +1307,7 @@ class HyperTreeListDemo(wx.Frame):
         sizera1.Add(self.unfocus, 0)
         sizera.Add(self.checknormal, 0, wx.ALL, 3)
         sizera.Add((1, 0), 1, wx.EXPAND)
-        sizera.Add(sizera1, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|wx.EXPAND, 3)
+        sizera.Add(sizera1, 0, wx.ALL|wx.EXPAND, 3)
 
         sizerb = wx.BoxSizer(wx.VERTICAL)
         self.checkgradient = wx.CheckBox(self.leftpanel, -1, "Gradient Theme")
@@ -1329,8 +1329,8 @@ class HyperTreeListDemo(wx.Frame):
                                               self.tree.GetSecondGradientColour())
         self.firstcolour.Bind(csel.EVT_COLOURSELECT, self.OnFirstColour)
         self.secondcolour.Bind(csel.EVT_COLOURSELECT, self.OnSecondColour)
-        sizerb3.Add(self.firstcolour, 0, wx.TOP|wx.BOTTOM|wx.ALIGN_CENTER_HORIZONTAL, 3)
-        sizerb3.Add(self.secondcolour, 0, wx.LEFT|wx.TOP|wx.BOTTOM|wx.ALIGN_CENTER_HORIZONTAL, 3)
+        sizerb3.Add(self.firstcolour, 0, wx.TOP|wx.BOTTOM, 3)
+        sizerb3.Add(self.secondcolour, 0, wx.LEFT|wx.TOP|wx.BOTTOM, 3)
         sizerb.Add(self.checkgradient, 0, wx.ALL, 3)
         sizerb.Add(sizerb1, 0)
         sizerb.Add(sizerb2, 0)
@@ -1339,10 +1339,14 @@ class HyperTreeListDemo(wx.Frame):
         self.checkvista = wx.CheckBox(self.leftpanel, -1, "Windows Vista Theme")
         self.checkvista.Bind(wx.EVT_CHECKBOX, self.OnVista)
 
+        self.dragFullScreen = wx.CheckBox(self.leftpanel, -1, "Fullscreen Drag/Drop")
+        self.dragFullScreen.Bind(wx.EVT_CHECKBOX, self.OnDragFullScreen)
+
         themessizer.Add(sizera, 0, wx.EXPAND)
         themessizer.Add(sizerb, 0, wx.EXPAND)
         themessizer.Add((0, 5))
         themessizer.Add(self.checkvista, 0, wx.EXPAND|wx.ALL, 3)
+        themessizer.Add(self.dragFullScreen, 0, wx.EXPAND|wx.ALL, 3)
 
         mainsizer.Add(stylesizer, 0, wx.EXPAND|wx.ALL, 5)
         mainsizer.Add(columnssizer, 0, wx.EXPAND|wx.ALL, 5)
@@ -1445,14 +1449,14 @@ class HyperTreeListDemo(wx.Frame):
 
             next = item
 
-            while next != None:
+            while next is not None:
 
                 self.tree.SetItemBackgroundColour(next, colour, col)
 
                 cookie=0
                 child, cockie = self.tree.GetNextChild(next, cookie)
 
-                while child != None:
+                while child is not None:
                     ColourItems(child, colour, col)
                     child, cookie = self.tree.GetNextChild(next, cookie)
 
@@ -1466,7 +1470,7 @@ class HyperTreeListDemo(wx.Frame):
         columnBackgroundColour = event.GetValue()
         selectedColumn = self.columnchoice.GetCurrentSelection()
         self.columnBackgroundColours[selectedColumn] = columnBackgroundColour
-        
+
         self.ColourColumnItems(columnBackgroundColour, selectedColumn)
 
         event.Skip()
@@ -1738,6 +1742,11 @@ class HyperTreeListDemo(wx.Frame):
 
         event.Skip()
 
+    def OnDragFullScreen(self, event):
+        
+        self.tree.SetDragFullScreen(event.IsChecked())
+        event.Skip()
+
 
 #---------------------------------------------------------------------------
 # HyperTreeList Implementation
@@ -1766,6 +1775,7 @@ class HyperTreeList(HTL.HyperTreeList):
         for evt in ["EVT_TREE_GET_INFO", "EVT_TREE_SET_INFO", "EVT_TREE_ITEM_MIDDLE_CLICK",
                     "EVT_TREE_STATE_IMAGE_CLICK"]:
             events.remove(evt)
+        events.extend(('EVT_LIST_COL_CLICK', 'EVT_LIST_COL_RIGHT_CLICK'))
 
         treestyles = treestyles + [i for i in dir(wx) if i.startswith("TR_")]
         treeset = {}
@@ -1776,6 +1786,8 @@ class HyperTreeList(HTL.HyperTreeList):
         self.events = events
         self.styles = treestyles
         self.item = None
+        self.sortIcon = wx.HDR_SORT_ICON_NONE
+        self.sortColumn = 0
 
         il = wx.ImageList(16, 16)
 
@@ -1906,7 +1918,8 @@ class HyperTreeList(HTL.HyperTreeList):
                           'EVT_TREE_ITEM_EXPANDING': self.OnItemExpanding, 'EVT_TREE_ITEM_GETTOOLTIP': self.OnToolTip,
                           'EVT_TREE_ITEM_MENU': self.OnItemMenu, 'EVT_TREE_ITEM_RIGHT_CLICK': self.OnRightDown,
                           'EVT_TREE_KEY_DOWN': self.OnKey, 'EVT_TREE_SEL_CHANGED': self.OnSelChanged,
-                          'EVT_TREE_SEL_CHANGING': self.OnSelChanging, "EVT_TREE_ITEM_HYPERLINK": self.OnHyperLink}
+                          'EVT_TREE_SEL_CHANGING': self.OnSelChanging, "EVT_TREE_ITEM_HYPERLINK": self.OnHyperLink,
+                          'EVT_LIST_COL_CLICK': self.OnColClick, 'EVT_LIST_COL_RIGHT_CLICK': self.OnColRightClick}
 
         mainframe = wx.GetTopLevelParent(self)
 
@@ -2100,7 +2113,7 @@ class HyperTreeList(HTL.HyperTreeList):
                          "itemtype": itemtype, "text": text, "pydata": pydata, "enabled": enabled}
 
         menu = wx.Menu()
-
+        item1 = menu.Append(wx.ID_ANY, "Change Item Text")
         item2 = menu.Append(wx.ID_ANY, "Modify Item Text Colour")
         menu.AppendSeparator()
         if isbold:
@@ -2135,6 +2148,7 @@ class HyperTreeList(HTL.HyperTreeList):
         item11 = menu.Append(wx.ID_ANY, "Prepend An Item")
         item12 = menu.Append(wx.ID_ANY, "Append An Item")
 
+        self.Bind(wx.EVT_MENU, self.OnItemText, item1)
         self.Bind(wx.EVT_MENU, self.OnItemForeground, item2)
         self.Bind(wx.EVT_MENU, self.OnItemBold, item3)
         self.Bind(wx.EVT_MENU, self.OnItemFont, item4)
@@ -2154,6 +2168,23 @@ class HyperTreeList(HTL.HyperTreeList):
         event.Skip()
 
 
+    def OnItemText(self, event):
+
+        col = wx.GetNumberFromUser("Choose column number to change text",
+                                   "Col", "Column To Modify", 0, 0,
+                                   self.GetColumnCount() - 1, self)
+        if col >= 0 and col < self.GetColumnCount():
+            diag = wx.TextEntryDialog(self, "%s Text" % self.GetColumnText(col),
+                                      caption="Input text for column %d" % col,
+                                      value=self.GetItemText(self.current,col),
+                                      style=wx.OK | wx.CANCEL | wx.TE_MULTILINE)
+            reply = diag.ShowModal()
+            text = diag.GetValue()
+            diag.Destroy()
+            if reply in (wx.OK, wx.ID_OK):
+                self.SetItemText(self.current, text, col)
+
+
     def OnItemForeground(self, event):
 
         colourdata = wx.ColourData()
@@ -2381,14 +2412,16 @@ class HyperTreeList(HTL.HyperTreeList):
     def OnItemExpanded(self, event):
 
         item = event.GetItem()
-        if item:
+        # Ignore wx.TreeCtrl TREE events (item=wx.TreeItemId) from item window.
+        if item and isinstance(item, HTL.TreeListItem):
             self.log.write("OnItemExpanded: %s\n" % self.GetItemText(item))
 
 
     def OnItemExpanding(self, event):
 
         item = event.GetItem()
-        if item:
+        # Ignore wx.TreeCtrl TREE events (item=wx.TreeItemId) from item window.
+        if item and isinstance(item, HTL.TreeListItem):
             self.log.write("OnItemExpanding: %s\n" % self.GetItemText(item))
 
         event.Skip()
@@ -2397,14 +2430,16 @@ class HyperTreeList(HTL.HyperTreeList):
     def OnItemCollapsed(self, event):
 
         item = event.GetItem()
-        if item:
+        # Ignore wx.TreeCtrl TREE events (item=wx.TreeItemId) from item window.
+        if item and isinstance(item, HTL.TreeListItem):
             self.log.write("OnItemCollapsed: %s" % self.GetItemText(item))
 
 
     def OnItemCollapsing(self, event):
 
         item = event.GetItem()
-        if item:
+        # Ignore wx.TreeCtrl TREE events (item=wx.TreeItemId) from item window.
+        if item and isinstance(item, HTL.TreeListItem):
             self.log.write("OnItemCollapsing: %s\n" % self.GetItemText(item))
 
         event.Skip()
@@ -2412,8 +2447,11 @@ class HyperTreeList(HTL.HyperTreeList):
 
     def OnSelChanged(self, event):
 
-        self.item = event.GetItem()
-        if self.item:
+        item = event.GetItem()
+        # Ignore wx.TreeCtrl TREE events (item=wx.TreeItemId) from item window.
+        if item and isinstance(item, HTL.TreeListItem):
+            # This event was from the HyperTreeList and not the TreeCtrl.
+            self.item = item
             self.log.write("OnSelChanged: %s" % self.GetItemText(self.item))
             if wx.Platform == '__WXMSW__':
                 self.log.write(", BoundingRect: %s\n" % self.GetBoundingRect(self.item, True))
@@ -2426,9 +2464,10 @@ class HyperTreeList(HTL.HyperTreeList):
     def OnSelChanging(self, event):
 
         item = event.GetItem()
-        olditem = event.GetOldItem()
-
-        if item:
+        # Ignore wx.TreeCtrl TREE events (item=wx.TreeItemId) from item window.
+        if item and isinstance(item, HTL.TreeListItem):
+            # This event was from the HyperTreeList and not the TreeCtrl.
+            olditem = event.GetOldItem()
             if not olditem:
                 olditemtext = "None"
             else:
@@ -2437,13 +2476,26 @@ class HyperTreeList(HTL.HyperTreeList):
 
         event.Skip()
 
+    def OnColClick(self, event):
+        column = event.GetColumn()
+        self.log.write("OnColClick: Column %d clicked" % column)
+        # Cycle through the sort icons.
+        if column != self.sortColumn:
+            self.sortIcon = wx.HDR_SORT_ICON_NONE
+            self.sortColumn = column
+        self.sortIcon = wx.HDR_SORT_ICON_DOWN if self.sortIcon == wx.HDR_SORT_ICON_UP else wx.HDR_SORT_ICON_UP
+        self.SetColumnSortIcon(column, self.sortIcon)
+
+    def OnColRightClick(self, event):
+        column = event.GetColumn()
+        self.log.write("OnColRightClick: Column %d clicked" % column)
+        self.SetColumnSortIcon(column, wx.HDR_SORT_ICON_NONE)
 
     def OnBeginDrag(self, event):
 
         self.item = event.GetItem()
         if self.item:
-            self.log.write("Beginning Drag...\n")
-
+            self.log.write("Beginning Drag... fullscreen=%s\n" % self.GetDragFullScreen())
             event.Allow()
 
 
@@ -2451,16 +2503,19 @@ class HyperTreeList(HTL.HyperTreeList):
 
         self.item = event.GetItem()
         if self.item:
-            self.log.write("Beginning Right Drag...\n")
-
+            self.log.write("Beginning Right Drag... fullscreen=%s\n" % self.GetDragFullScreen())
             event.Allow()
 
 
     def OnEndDrag(self, event):
 
-        self.item = event.GetItem()
-        if self.item:
-            self.log.write("Ending Drag!\n")
+        if self.GetDragFullScreen() is True:
+            wnd = wx.FindWindowAtPoint(self.ClientToScreen(event.GetPoint()))
+            self.log.write("Ending Drag! window=%s\n" % repr(wnd))
+        else:
+            self.item = event.GetItem()
+            name = self.GetItemText(self.item) if self.item else 'None'
+            self.log.write("Ending Drag! item=%s\n" % name)
 
         event.Skip()
 
@@ -2479,7 +2534,7 @@ class HyperTreeList(HTL.HyperTreeList):
     def OnItemCheck(self, event):
 
         item = event.GetItem()
-        self.log.write("Item " + self.GetItemText(item) + " Has Been Cheched!\n")
+        self.log.write("Item " + self.GetItemText(item) + " Has Been Checked!\n")
         event.Skip()
 
 
