@@ -3,7 +3,7 @@
 # Author:      Robin Dunn
 #
 # Created:     22-Nov-2010
-# Copyright:   (c) 2010-2018 by Total Control Software
+# Copyright:   (c) 2010-2020 by Total Control Software
 # License:     wxWindows License
 #---------------------------------------------------------------------------
 
@@ -67,6 +67,8 @@ def run():
     c.find('OnExceptionInMainLoop').ignore()
     c.find('OnFatalException').ignore()
     c.find('OnUnhandledException').ignore()
+    c.find('StoreCurrentException').ignore()
+    c.find('RethrowStoredException').ignore()
 
     # Release the GIL for potentially blocking or long-running functions
     c.find('MainLoop').releaseGIL()
@@ -121,19 +123,6 @@ def run():
                 return 0;
             #endif
             """)
-    c.addCppMethod('int', 'GetShell32Version', '()',
-        isStatic=True,
-        doc="""\
-        Returns 400, 470, 471, etc. for shell32.dll 4.00, 4.70, 4.71 or 0 if
-        it wasn't found at all.  Raises an exception on non-Windows platforms.""",
-        body="""\
-            #ifdef __WXMSW__
-                return wxApp::GetShell32Version();
-            #else
-                wxPyRaiseNotImplemented();
-                return 0;
-            #endif
-            """)
 
 
     # Remove the virtualness from these methods
@@ -217,9 +206,27 @@ def run():
     c.addProperty('UseBestVisual GetUseBestVisual SetUseBestVisual')
     c.addProperty('TopWindow GetTopWindow SetTopWindow')
 
+    c.find('GTKSuppressDiagnostics').setCppCode("""\
+        #ifdef __WXGTK__
+            wxApp::GTKSuppressDiagnostics(flags);
+        #endif
+        """)
+
+    c.find('GetGUIInstance').ignore()
+
 
     #-------------------------------------------------------
 
+    module.find('wxHandleFatalExceptions').setCppCode("""\
+        #if wxUSE_ON_FATAL_EXCEPTION
+            return wxHandleFatalExceptions(doIt);
+        #else
+            wxLogInfo("This build of wxWidgets does not support wxHandleFatalExceptions.");
+            return false;
+        #endif
+        """)
+
+    #-------------------------------------------------------
 
     module.addHeaderCode("""\
         enum wxAppAssertMode {
@@ -350,7 +357,12 @@ def run():
 
             Normally you would derive from this class and implement an ``OnInit``
             method that creates a frame and then calls ``self.SetTopWindow(frame)``,
-            however ``wx.App`` is also usable on it's own without derivation.
+            however ``wx.App`` is also usable on its own without derivation.
+
+            :note: In Python the wrapper for the C++ class ``wxApp`` has been renamed tp
+                :class:`wx.PyApp`. This ``wx.App`` class derives from ``wx.PyApp``, and is
+                responsible for handling the Python-specific needs for bootstrapping the
+                wxWidgets library and other Python integration related requirements.
             """,
 
         items=[
@@ -517,31 +529,28 @@ def run():
 
             PyFunctionDef('InitLocale', '(self)',
                 doc="""\
-                    Try to ensure that the C and Python locale is in sync with wxWidgets locale.
+                    Starting with version 3.8 on Windows, Python is now setting the locale
+                    to what is defined by the system as the default locale. This causes
+                    problems with wxWidgets which expects to be able to manage the locale
+                    via the wx.Locale class, so the locale will be reset here to be the
+                    default "C" locale settings.
+
+                    If you have troubles from the default behavior of this method you can
+                    override it in a derived class to behave differently. Please report
+                    the problem you encountered.
                     """,
                 body="""\
-                    self.ResetLocale()
-                    import locale
-                    try:
-                        loc, enc = locale.getlocale()
-                    except ValueError:
-                        loc = enc = None
-                    # Try to set it to the same language as what is already set in the C locale
-                    info = wx.Locale.FindLanguageInfo(loc) if loc else None
-                    if info:
-                        self._initial_locale = wx.Locale(info.Language)
-                    else:
-                        # otherwise fall back to the system default
-                        self._initial_locale = wx.Locale(wx.LANGUAGE_DEFAULT)
+                    if _sys.platform.startswith('win') and _sys.version_info > (3,8):
+                        import locale
+                        locale.setlocale(locale.LC_ALL, "C")
                     """),
 
             PyFunctionDef('ResetLocale', '(self)',
                 doc="""\
-                    Release the wx.Locale object created in :meth:`InitLocale`.
-                    This will reset the application's locale to the previous settings.
+                    This method is now a NOP and will be deprecated.
                     """,
                 body="""\
-                    self._initial_locale = None
+                    pass
                     """),
 
             PyFunctionDef('Get', '()', isStatic=True,
