@@ -3,7 +3,7 @@
 # Author:      Robin Dunn
 #
 # Created:     15-Nov-2010
-# Copyright:   (c) 2010-2018 by Total Control Software
+# Copyright:   (c) 2010-2020 by Total Control Software
 # License:     wxWindows License
 #---------------------------------------------------------------------------
 
@@ -33,6 +33,7 @@ ITEMS  = [
     'wxCloseEvent',
     'wxContextMenuEvent',
     'wxDisplayChangedEvent',
+    'wxDPIChangedEvent',
     'wxDropFilesEvent',
     'wxEraseEvent',
     'wxFocusEvent',
@@ -43,6 +44,7 @@ ITEMS  = [
     'wxJoystickEvent',
     'wxKeyEvent',
     'wxMaximizeEvent',
+    'wxFullScreenEvent',
     'wxMenuEvent',
     'wxMouseCaptureChangedEvent',
     'wxMouseCaptureLostEvent',
@@ -63,6 +65,15 @@ ITEMS  = [
     'wxWindowCreateEvent',
     'wxWindowDestroyEvent',
 
+    'wxGestureEvent',
+    'wxPanGestureEvent',
+    'wxZoomGestureEvent',
+    'wxRotateGestureEvent',
+    'wxTwoFingerTapEvent',
+    'wxLongPressEvent',
+    'wxPressAndTapEvent',
+
+
     #'wxThreadEvent',
 
 ]
@@ -89,15 +100,6 @@ def run():
     #endif
     """)
 
-    # C macros that need to be ignored
-    module.find('wx__DECLARE_EVT0').ignore()
-    module.find('wx__DECLARE_EVT1').ignore()
-    module.find('wx__DECLARE_EVT2').ignore()
-    module.find('wxEVENT_HANDLER_CAST').ignore()
-    module.find('wxDECLARE_EXPORTED_EVENT').ignore()
-    module.find('wxDECLARE_EVENT').ignore()
-    module.find('wxDEFINE_EVENT').ignore()
-
 
     module.addPyClass('PyEventBinder', ['object'],
         doc="""\
@@ -221,10 +223,10 @@ def run():
                 // pointer and disconnect that one.  Unfortunately since we
                 // wrapped the PyObject function pointer in another object we
                 // have to do the searching ourselves...
-                wxList::compatibility_iterator node = self->GetDynamicEventTable()->GetFirst();
-                while (node)
+                size_t cookie;
+                wxDynamicEventTableEntry *entry = self->GetFirstDynamicEntry(cookie);
+                while (entry)
                 {
-                    wxDynamicEventTableEntry *entry = (wxDynamicEventTableEntry*)node->GetData();
                     if ((entry->m_id == id) &&
                         ((entry->m_lastId == lastId) || (lastId == wxID_ANY)) &&
                         ((entry->m_eventType == eventType) || (eventType == wxEVT_NULL)) &&
@@ -238,12 +240,16 @@ def run():
                         // is evaluated. (!!!)
                         if (PyObject_RichCompareBool(cb->m_func, func, Py_EQ) == 1) {
                             delete cb;
-                            self->GetDynamicEventTable()->Erase(node);
-                            delete entry;
-                            return true;
+                            // Set callback data to a known value instead of NULL to
+                            // ensure Disconnect() removes the correct handler.
+                            entry->m_callbackUserData = new wxObject();
+                            // Now Disconnect should work
+                            return self->Disconnect(id, lastId, eventType,
+                                                    (wxObjectEventFunction)&wxPyCallback::EventThunker,
+                                                    entry->m_callbackUserData);
                         }
                     }
-                    node = node->GetNext();
+                    entry = self->GetNextDynamicEntry(cookie);
                 }
                 return false;
             }
@@ -260,10 +266,6 @@ def run():
     for m in c.find('CallAfter').all():
         m.ignore()
 
-    # wxEventTable is not documented so we have to ignore SearchEventTable.
-    # TODO: Should wxEventTable be available to language bindings?
-    c.find('SearchEventTable').ignore()
-
     c.find('QueueEvent.event').transfer = True
     module.find('wxQueueEvent.event').transfer = True
 
@@ -347,6 +349,8 @@ def run():
     c.abstract = True
     c.find('Clone').factory = True
 
+    c.find('GetEventUserData').ignore()
+
     c.addProperty('EventObject GetEventObject SetEventObject')
     c.addProperty('EventType GetEventType SetEventType')
     c.addProperty('Id GetId SetId')
@@ -383,6 +387,14 @@ def run():
     c.addProperty('String GetString SetString')
 
 
+    #---------------------------------------
+    # wxPaintEvent
+    c = module.find('wxPaintEvent')
+    # Although the default ctor is listed as public in the interface, it is
+    # magically made private for the users of the library as it can only be
+    # created within wxWidgets.
+    c.find('wxPaintEvent').protection = 'private'
+
     #---------------------------------------
     # wxKeyEvent
     c = module.find('wxKeyEvent')
@@ -390,13 +402,25 @@ def run():
     c.find('GetPosition').findOverload('wxCoord').ignore()
     c.find('GetUnicodeKey').type = 'int'
 
+    c.addCppMethod('void', 'SetKeyCode', '(int keyCode)',
+        body="self->m_keyCode = keyCode;")
+
+    c.addCppMethod('void', 'SetRawKeyCode', '(int rawKeyCode)',
+        body="self->m_rawCode = rawKeyCode;")
+
+    c.addCppMethod('void', 'SetRawKeyFlags', '(int rawFlags)',
+        body="self->m_rawFlags = rawFlags;")
+
+    c.addCppMethod('void', 'SetUnicodeKey', '(int uniChar)',
+        body="self->m_uniChar = uniChar;")
+
     c.addProperty('X GetX')
     c.addProperty('Y GetY')
-    c.addProperty('KeyCode GetKeyCode')
+    c.addProperty('KeyCode GetKeyCode SetKeyCode')
     c.addProperty('Position GetPosition')
-    c.addProperty('RawKeyCode GetRawKeyCode')
-    c.addProperty('RawKeyFlags GetRawKeyFlags')
-    c.addProperty('UnicodeKey GetUnicodeKey')
+    c.addProperty('RawKeyCode GetRawKeyCode SetRawKeyCode')
+    c.addProperty('RawKeyFlags GetRawKeyFlags SetRawKeyFlags')
+    c.addProperty('UnicodeKey GetUnicodeKey SetUnicodeKey')
 
     #---------------------------------------
     # wxScrollEvent
@@ -587,7 +611,10 @@ def run():
 
     #---------------------------------------
     # wxIconizeEvent
-    module.find('wxIconizeEvent.Iconized').deprecated = True
+    c = module.find('wxIconizeEvent')
+    # deprecated and removed
+    c.find('Iconized').ignore()
+
 
 
     # Apply common fixups for all the event classes
