@@ -3,7 +3,7 @@
 # Author:      Robin Dunn
 #
 # Created:     19-Dec-2010
-# Copyright:   (c) 2010-2018 by Total Control Software
+# Copyright:   (c) 2010-2020 by Total Control Software
 # License:     wxWindows License
 #---------------------------------------------------------------------------
 
@@ -63,6 +63,13 @@ def run():
     module.find('wxSetDisplayName').ignore()
     module.find('wxPostDelete').ignore()
 
+    # TODO: Can this be added back? It was removed because of compiler errors.
+    module.find('wxGetDiskSpace').ignore()
+
+    # deprecated and removed
+    module.find('wxUsleep').ignore()
+
+
     # ignore all the environment related functions
     for item in module.allItems():
         if 'Env' in item.name:
