@@ -14,9 +14,34 @@
     wxTreeItemId handles, which may be tested for validity by calling
     wxTreeItemId::IsOk().
 
-    A similar control with a fully native implementation for GTK+ and OS X
+    A similar control with a fully native implementation for GTK+ and macOS
     as well is wxDataViewTreeCtrl.
 
+    @section treectrl_images Images in wxTreeCtrl
+
+    wxTreeCtrl inherits from wxWithImages classes providing the functions for
+    associating images with the control items. Each item refers to its image
+    using an index, which can possibly by wxWithImages::NO_IMAGE to indicate
+    that the item doesn't use any image at all, and the corresponding image is
+    taken either from the vector passed to wxWithImages::SetImages() or from
+    the image list passed to wxWithImages::SetImageList() or
+    wxWithImages::AssignImageList() functions.
+
+    In addition to normal images, handled with the methods mentioned above,
+    wxTreeCtrl also provides optional state images that may be used to indicate
+    some additional state of the item, e.g. checked or unchecked status. These
+    images can be set using SetStateImageList() and AssignStateImageList()
+    functions that behave in the same way as the corresponding methods of
+    wxWithImages.
+
+    Finally, in the generic version of this control (wxGenericTreeCtrl), also
+    provides SetButtonsImageList() and AssignButtonsImageList(), which can be
+    used to change the images used for the control buttons, used to expand or
+    collapse its branches. These methods are not available in the native wxMSW
+    and wxQt implementations.
+
+    @section treectrl_events Events
+
     To intercept events from a tree control, use the event table macros
     described in wxTreeEvent.
 
@@ -32,10 +57,8 @@
         Selects alternative style of @c +/@c - buttons and shows rotating
         ("twisting") arrows instead. Currently this style is only implemented
         under Microsoft Windows Vista and later Windows versions and is ignored
-        under the other platforms. Notice that under Vista this style results
-        in the same appearance as used by the tree control in Explorer and
-        other built-in programs and so using it may be preferable to the
-        default style.
+        under the other platforms as enabling it is equivalent to using
+        wxSystemThemedControl::EnableSystemTheme().
     @style{wxTR_NO_LINES}
         Use this style to hide vertical level connectors.
     @style{wxTR_FULL_ROW_HIGHLIGHT}
@@ -44,8 +67,11 @@
         flag is ignored under Windows unless you specify @c wxTR_NO_LINES as
         well.)
     @style{wxTR_LINES_AT_ROOT}
-        Use this style to show lines between root nodes. Only applicable if @c
-        wxTR_HIDE_ROOT is set and @c wxTR_NO_LINES is not set.
+        Use this style to show lines leading to the root nodes (unless no
+        @c wxTR_NO_LINES is also used, in which case no lines are shown). Note
+        that in the MSW version, if this style is omitted, not only the lines,
+        but also the button used for expanding the root item is not shown,
+        which can be unexpected, so it is recommended to always use it.
     @style{wxTR_HIDE_ROOT}
         Use this style to suppress the display of the root node, effectively
         causing the first-level nodes to appear as a series of root nodes.
@@ -134,7 +160,10 @@
           Processes a @c wxEVT_TREE_ITEM_GETTOOLTIP event type.
     @event{EVT_TREE_ITEM_MENU(id, func)}
           The context menu for the selected item has been requested, either by a
-          right click or by using the menu key.
+          right click or by using the menu key. Notice that these events always
+          carry a valid tree item and so are not generated when (right)
+          clicking outside of the items area. If you need to handle such
+          events, consider using @c wxEVT_CONTEXT_MENU instead.
           Processes a @c wxEVT_TREE_ITEM_MENU event type.
     @event{EVT_TREE_STATE_IMAGE_CLICK(id, func)}
           The state image has been clicked.
@@ -160,9 +189,9 @@
     @appearance{treectrl}
 
     @see wxDataViewTreeCtrl, wxTreeEvent, wxTreeItemData, @ref overview_treectrl,
-         wxListBox, wxListCtrl, wxImageList
+         wxListBox, wxListCtrl, wxWithImages
 */
-class wxTreeCtrl : public wxControl
+class wxTreeCtrl : public wxControl, public wxWithImages
 {
 public:
     /**
@@ -181,7 +210,7 @@ public:
             Window position.
             If ::wxDefaultPosition is specified then a default position is chosen.
         @param size
-            Window size. 
+            Window size.
             If ::wxDefaultSize is specified then the window is sized appropriately.
         @param style
             Window style. See wxTreeCtrl.
@@ -210,7 +239,7 @@ public:
 
         The @a image and @a selImage parameters are an index within the normal
         image list specifying the image to use for unselected and selected
-        items, respectively. If @a image -1 and @a selImage is -1, the same
+        items, respectively. If @a image > -1 and @a selImage is -1, the same
         image is used for both selected and unselected items.
     */
     virtual wxTreeItemId AddRoot(const wxString& text, int image = -1,
@@ -247,15 +276,6 @@ public:
     */
     void AssignButtonsImageList(wxImageList* imageList);
 
-    /**
-        Sets the normal image list. The image list assigned with this method
-        will be automatically deleted by wxTreeCtrl as appropriate (i.e. it
-        takes ownership of the list).
-
-        @see SetImageList().
-    */
-    void AssignImageList(wxImageList* imageList);
-
     /**
         Sets the state image list. Image list assigned with this method will be
         automatically deleted by wxTreeCtrl as appropriate (i.e. it takes
@@ -309,15 +329,19 @@ public:
     virtual void Delete(const wxTreeItemId& item);
 
     /**
-        Deletes all items in the control. Note that this may not generate
-        @c EVT_TREE_DELETE_ITEM events under some Windows versions although
-        normally such event is generated for each removed item.
+        Deletes all items in the control.
+
+        This function generates @c wxEVT_TREE_DELETE_ITEM events for each item
+        being deleted, including the root one if it is shown, i.e. unless
+        wxTR_HIDE_ROOT style is used.
     */
     virtual void DeleteAllItems();
 
     /**
-        Deletes all children of the given item (but not the item itself). Note
-        that this will @b not generate any events unlike Delete() method.
+        Deletes all children of the given item (but not the item itself).
+
+        A @c wxEVT_TREE_DELETE_ITEM event will be generated for every item
+        being deleted.
 
         If you have called SetItemHasChildren(), you may need to call it again
         since DeleteChildren() does not automatically clear the setting.
@@ -363,6 +387,9 @@ public:
 
     /**
         Scrolls and/or expands items to ensure that the given item is visible.
+
+        This method can be used, and will work, even while the window is frozen
+        (see wxWindow::Freeze()).
     */
     virtual void EnsureVisible(const wxTreeItemId& item);
 
@@ -462,7 +489,7 @@ public:
         Returns the item last clicked or otherwise selected.
         Unlike GetSelection(), it can be used whether or not
         the control has the @c wxTR_MULTIPLE style.
-        
+
         @since 2.9.1
     */
     virtual wxTreeItemId GetFocusedItem() const;
@@ -484,17 +511,17 @@ public:
     */
     virtual void SetFocusedItem(const wxTreeItemId& item);
 
-
-    /**
-        Returns the normal image list.
-    */
-    wxImageList* GetImageList() const;
-
     /**
         Returns the current tree control indentation.
     */
     virtual unsigned int GetIndent() const;
 
+    /**
+        Returns the current tree control spacing.  This is the number of
+        horizontal pixels between the buttons and the state images.
+    */
+    unsigned int GetSpacing() const;
+
     /**
         Returns the background colour of the item.
     */
@@ -639,7 +666,7 @@ public:
     /**
         Returns the selection, or an invalid item if there is no selection. This
         function only works with the controls without @c wxTR_MULTIPLE style,
-        use GetSelections() for the controls which do have this style 
+        use GetSelections() for the controls which do have this style
         or, if a single item is wanted, use GetFocusedItem().
     */
     virtual wxTreeItemId GetSelection() const;
@@ -695,7 +722,7 @@ public:
 
         The @a image and @a selImage parameters are an index within the normal
         image list specifying the image to use for unselected and selected
-        items, respectively. If @a image -1 and @a selImage is -1, the same
+        items, respectively. If @a image > -1 and @a selImage is -1, the same
         image is used for both selected and unselected items.
     */
     wxTreeItemId InsertItem(const wxTreeItemId& parent,
@@ -712,7 +739,7 @@ public:
 
         The @a image and @a selImage parameters are an index within the normal
         image list specifying the image to use for unselected and selected
-        items, respectively. If @a image -1 and @a selImage is -1, the same
+        items, respectively. If @a image > -1 and @a selImage is -1, the same
         image is used for both selected and unselected items.
     */
     wxTreeItemId InsertItem(const wxTreeItemId& parent,
@@ -778,7 +805,7 @@ public:
 
         The @a image and @a selImage parameters are an index within the normal
         image list specifying the image to use for unselected and selected
-        items, respectively. If @a image -1 and @a selImage is -1, the same
+        items, respectively. If @a image > -1 and @a selImage is -1, the same
         image is used for both selected and unselected items.
     */
     wxTreeItemId PrependItem(const wxTreeItemId& parent,
@@ -789,6 +816,11 @@ public:
 
     /**
         Scrolls the specified item into view.
+
+        Note that this method doesn't work while the window is frozen (See
+        wxWindow::Freeze()), at least under MSW.
+
+        @see EnsureVisible()
     */
     virtual void ScrollTo(const wxTreeItemId& item);
 
@@ -820,20 +852,18 @@ public:
     */
     void SetButtonsImageList(wxImageList* imageList);
 
-    /**
-        Sets the normal image list. The image list assigned with this method
-        will @b not be deleted by @ref wxTreeCtrl "wxTreeCtrl"'s destructor, you
-        must delete it yourself.
-
-        @see AssignImageList().
-    */
-    virtual void SetImageList(wxImageList* imageList);
-
     /**
         Sets the indentation for the tree control.
     */
     virtual void SetIndent(unsigned int indent);
 
+    /**
+        Sets the spacing for the tree control. Spacing is the number of
+        horizontal pixels between the buttons and the state images.
+        This has no effect under wxMSW.
+    */
+    void SetSpacing(unsigned int spacing);
+
     /**
         Sets the colour of the item's background.
     */
@@ -1075,7 +1105,12 @@ public:
                 const wxTreeItemId& item = wxTreeItemId());
 
     /**
-        Returns the item (valid for all events).
+        Returns the item.
+
+        Note that the item may be invalid for wxEVT_TREE_SEL_CHANGED events
+        when the previously selected item has been deselected and there is no
+        new selection any longer, as it notably happens when deleting all tree
+        control items.
     */
     wxTreeItemId GetItem() const;
 
@@ -1096,8 +1131,8 @@ public:
     const wxString& GetLabel() const;
 
     /**
-        Returns the old item index (valid for @c EVT_TREE_ITEM_CHANGING and
-        @c EVT_TREE_ITEM_CHANGED events).
+        Returns the old item index (valid for @c EVT_TREE_SEL_CHANGING and
+        @c EVT_TREE_SEL_CHANGED events).
     */
     wxTreeItemId GetOldItem() const;
 
