@@ -58,13 +58,16 @@ enum wxShutdownFlags
 /**
     @class wxWindowDisabler
 
-    This class disables all windows of the application (may be with the
-    exception of one of them) in its constructor and enables them back in its
-    destructor.
+    This class disables all top level windows of the application (maybe with
+    the exception of one of them) in its constructor and enables them back in
+    its destructor.
 
     This is useful when you want to indicate to the user that the application
     is currently busy and cannot respond to user input.
 
+    @note When instantiated, this affects only windows shown on the screen and
+          not already disabled.
+
     @library{wxcore}
     @category{misc}
 
@@ -81,7 +84,7 @@ public:
 
         @since 2.9.0
     */
-    wxWindowDisabler(bool disable = true);
+    explicit wxWindowDisabler(bool disable = true);
 
     /**
         Disables all top level windows of the applications with the exception
@@ -94,8 +97,12 @@ public:
         from happening you may want to use wxFRAME_TOOL_WINDOW, if applicable,
         or wxFRAME_NO_TASKBAR style when creating the window that will remain
         enabled.
+
+        The argument @a winToSkip2 may be used to specify another window to
+        leave enabled, if it is non-null. This parameter is only available
+        since wxWidgets 3.1.7.
     */
-    wxWindowDisabler(wxWindow* winToSkip);
+    explicit wxWindowDisabler(wxWindow* winToSkip, wxWindow* winToSkip2 = NULL);
 
     /**
         Reenables the windows disabled by the constructor.
@@ -332,6 +339,23 @@ bool wxGetEnvMap(wxEnvVariableHashMap *map);
 /** @addtogroup group_funcmacro_misc */
 //@{
 
+/**
+    Fills the memory block with zeros in a way that is guaranteed
+    not to be optimized away by the compiler.
+
+    @param p Pointer to the memory block to be zeroed, must be non-@NULL.
+    @param n The number of bytes to zero.
+
+    NOTE: If security is vitally important in your use case, please
+    have a look at the implementations and decide whether you trust
+    them to behave as promised.
+
+    @header{wx/utils.h}
+
+    @since 3.1.6
+*/
+void wxSecureZeroMemory(void *p, size_t n);
+
 /**
     Returns battery state as one of @c wxBATTERY_NORMAL_STATE,
     @c wxBATTERY_LOW_STATE, @c wxBATTERY_CRITICAL_STATE,
@@ -362,6 +386,26 @@ wxPowerType wxGetPowerType();
 */
 wxString wxGetDisplayName();
 
+/**
+    This function returns the total number of bytes and number of free bytes on
+    the disk containing the directory @a path (it should exist). Both @a total
+    and @a free parameters may be @NULL if the corresponding information is not
+    needed.
+
+    @since 2.3.2
+
+    @note The generic Unix implementation depends on the system having the
+          @c statfs() or @c statvfs() function.
+
+    @return @true on success, @false if an error occurred (for example, the
+             directory doesn’t exist).
+
+    @header{wx/utils.h}
+*/
+bool wxGetDiskSpace(const wxString& path,
+                    wxLongLong total = NULL,
+                    wxLongLong free = NULL);
+
 /**
     For normal keys, returns @true if the specified key is currently down.
 
@@ -459,9 +503,11 @@ int wxFindMenuItemId(wxFrame* frame, const wxString& menuString,
 
     Generates an integer identifier unique to this run of the program.
 
+    @see wxRegisterId()
+
     @header{wx/utils.h}
 */
-int wxNewId();
+wxWindowID wxNewId();
 
 /**
     Ensures that Ids subsequently generated by wxNewId() do not clash with the
@@ -469,7 +515,7 @@ int wxNewId();
 
     @header{wx/utils.h}
 */
-void wxRegisterId(int id);
+void wxRegisterId(wxWindowID id);
 
 /**
     Opens the @a document in the application associated with the files of this
@@ -659,14 +705,54 @@ void wxSetDisplayName(const wxString& displayName);
 */
 enum
 {
-    // strip '&' characters
+    /**
+        Strip '&' characters.
+
+        This flag removes all the ampersands before another character and
+        replaces double ampersands with a single one.
+     */
     wxStrip_Mnemonics = 1,
 
-    // strip everything after '\t'
+    /**
+        Strip everything after '\\t'.
+
+        This flags removes everything following the last TAB character in the
+        string, if any.
+     */
     wxStrip_Accel = 2,
 
-    // strip everything (this is the default)
-    wxStrip_All = wxStrip_Mnemonics | wxStrip_Accel
+    /**
+        Strip everything looking like CJK mnemonic.
+
+        CJK (Chinese, Japanese, Korean) translations sometimes preserve the
+        original English accelerator or mnemonic in the translated string by
+        putting it after the translated string in parentheses, e.g. the string
+        "&File" could be translated as "<translation-of-word-file> (&F)".
+
+        This flag strips trailing "(&X)" from the string.
+
+        @since 3.1.3
+     */
+    wxStrip_CJKMnemonics = 4,
+
+    /**
+        Strip both mnemonics and accelerators.
+
+        This is the value used by wxStripMenuCodes() by default.
+
+        Note that, despite the name, this flag does @e not strip all, as it
+        doesn't include wxStrip_CJKMnemonics for compatibility.
+     */
+    wxStrip_All = wxStrip_Mnemonics | wxStrip_Accel,
+
+    /**
+        Strip everything from menu item labels.
+
+        This flag is used by wxWidgets internally and removes CJK mnemonics
+        from the labels too, in addition to the usual mnemonics and
+        accelerators. It is only suitable for use with the menu items.
+     */
+    wxStrip_Menu = wxStrip_All | wxStrip_CJKMnemonics
 };
 
 /**
@@ -736,10 +822,6 @@ wxString wxGetHomeDir();
     note that the returned name is @e not fully qualified, i.e. it does not
     include the domain name.
 
-    Under Windows or NT, this function first looks in the environment variable
-    SYSTEM_NAME; if this is not found, the entry @b HostName in the wxWidgets
-    section of the WIN.INI file is tried.
-
     @return The hostname if successful or an empty string otherwise.
 
     @see wxGetFullHostName()
@@ -838,7 +920,7 @@ bool wxGetUserName(char* buf, int sz);
 /**
     Returns the string containing the description of the current platform in a
     user-readable form. For example, this function may return strings like
-    "Windows NT Version 4.0" or "Linux 2.2.2 i386".
+    "Windows 10 (build 10240), 64-bit edition" or "Linux 4.1.4 i386".
 
     @see wxGetOsVersion()
 
@@ -847,32 +929,126 @@ bool wxGetUserName(char* buf, int sz);
 wxString wxGetOsDescription();
 
 /**
-    Gets the version and the operating system ID for currently running OS. 
+    Gets the version and the operating system ID for currently running OS.
     The returned wxOperatingSystemId value can be used for a basic categorization
-    of the OS family; the major and minor version numbers allows detecting a specific
-    system.
-    
-    For Unix-like systems (@c wxOS_UNIX) the major and minor version integers will 
-    contain the kernel major and minor version numbers (as returned by the
-    'uname -r' command); e.g. "2" and "6" if the machine is using kernel 2.6.19.
+    of the OS family; the major, minor, and micro version numbers allows
+    detecting a specific system.
 
-    For Mac OS X systems (@c wxOS_MAC) the major and minor version integers are the
-    natural version numbers associated with the OS; e.g. "10" and "6" if the machine
-    is using Mac OS X Snow Leopard.    
-    
-    For Windows-like systems (@c wxOS_WINDOWS) the major and minor version integers will 
-    contain the following values:
-    @beginTable
-    @row3col{<b>Windows OS name</b>, <b>Major version</b>, <b>Minor version</b>}
-    @row3col{Windows 7,                 6, 1}
-    @row3col{Windows Server 2008 R2,    6, 1}
-    @row3col{Windows Server 2008,       6, 0}
-    @row3col{Windows Vista,             6, 0}
-    @row3col{Windows Server 2003 R2,    5, 2}
-    @row3col{Windows Server 2003,       5, 2}
-    @row3col{Windows XP,                5, 1}
-    @row3col{Windows 2000,              5, 0}
-    @endDefList
+    If on Unix-like systems the version can't be detected all three version
+    numbers will have a value of -1.
+
+    On systems where only the micro version can't be detected or doesn't make
+    sense, it will have a value of 0.
+
+    For Unix-like systems (@c wxOS_UNIX) the major, minor, and micro version
+    integers will contain the kernel's major, minor, and micro version
+    numbers (as returned by the 'uname -r' command); e.g. "4", "1", and "4" if
+    the machine is using kernel 4.1.4.
+
+    For macOS systems (@c wxOS_MAC) the major and minor version integers are the
+    natural version numbers associated with the OS; e.g. "10", "11" and "2" if
+    the machine is using macOS El Capitan 10.11.2.
+
+    For Windows-like systems (@c wxOS_WINDOWS) the major, minor and micro
+    (equal to the build number) version integers will contain the following values:
+    <table>
+        <tr>
+            <th>Windows OS name</th>
+            <th>Major version</th>
+            <th>Minor version</th>
+            <th>Build number</th>
+        </tr>
+        <tr>
+            <td>Windows 11</td>
+            <td>10</td>
+            <td>0</td>
+            <td>&gt;= 22000</td>
+        </tr>
+        <tr>
+            <td>Windows Server 2022</td>
+            <td>10</td>
+            <td>0</td>
+            <td>&gt;= 22000</td>
+        </tr>
+        <tr>
+            <td>Windows 10</td>
+            <td>10</td>
+            <td>0</td>
+            <td></td>
+        </tr>
+        <tr>
+            <td>Windows Server 2016</td>
+            <td>10</td>
+            <td>0</td>
+            <td></td>
+        </tr>
+        <tr>
+            <td>Windows 8.1</td>
+            <td>6</td>
+            <td>3</td>
+            <td></td>
+        </tr>
+        <tr>
+            <td>Windows Server 2012 R2</td>
+            <td>6</td>
+            <td>3</td>
+            <td></td>
+        </tr>
+        <tr>
+            <td>Windows 8</td>
+            <td>6</td>
+            <td>2</td>
+            <td></td>
+        </tr>
+        <tr>
+            <td>Windows Server 2012</td>
+            <td>6</td>
+            <td>2</td>
+            <td></td>
+        </tr>
+        <tr>
+            <td>Windows 7</td>
+            <td>6</td>
+            <td>1</td>
+            <td></td>
+        </tr>
+        <tr>
+            <td>Windows 2008 R2</td>
+            <td>6</td>
+            <td>1</td>
+            <td></td>
+        </tr>
+        <tr>
+            <td>Windows Vista</td>
+            <td>6</td>
+            <td>0</td>
+            <td></td>
+        </tr>
+        <tr>
+            <td>Windows Server 2008</td>
+            <td>6</td>
+            <td>0</td>
+            <td></td>
+        </tr>
+        <tr>
+            <td>Windows Server 2003 R2</td>
+            <td>5</td>
+            <td>2</td>
+            <td></td>
+        </tr>
+        <tr>
+            <td>Windows Server 2003</td>
+            <td>5</td>
+            <td>2</td>
+            <td></td>
+        </tr>
+        <tr>
+            <td>Windows XP</td>
+            <td>5</td>
+            <td>1</td>
+            <td></td>
+        </tr>
+    </table>
     See the <a href="http://msdn.microsoft.com/en-us/library/ms724832(VS.85).aspx">MSDN</a>
     for more info about the values above.
 
@@ -880,19 +1056,30 @@ wxString wxGetOsDescription();
 
     @header{wx/utils.h}
 */
-wxOperatingSystemId wxGetOsVersion(int* major = NULL, int* minor = NULL);
+wxOperatingSystemId wxGetOsVersion(int* major = NULL, int* minor = NULL, int* micro = NULL);
+
+/**
+    Returns @true if the version of the operating system on which the program
+    is running under is the same or later than the given version.
+
+    @since 3.1.0
+
+    @see wxGetOsVersion(), wxPlatformInfo
+
+    @header{wx/utils.h}
+*/
+bool wxCheckOsVersion(int majorVsn, int minorVsn = 0, int microVsn = 0);
 
 /**
     Returns @true if the operating system the program is running under is 64
     bit. The check is performed at run-time and may differ from the value
     available at compile-time (at compile-time you can just check if
     <tt>sizeof(void*) == 8</tt>) since the program could be running in
-    emulation mode or in a mixed 32/64 bit system (bi-architecture operating
-    system).
+    emulation mode or in a mixed 32/64 bit system.
 
     @note This function is not 100% reliable on some systems given the fact
           that there isn't always a standard way to do a reliable check on the
-          OS architecture.
+          OS bitness.
 
     @header{wx/utils.h}
 */
@@ -908,18 +1095,51 @@ bool wxIsPlatform64Bit();
 */
 bool wxIsPlatformLittleEndian();
 
+/**
+    Returns the CPU architecture name. This can be, for example, "x86_64",
+    "arm64", or "i86pc". The name for the same CPU running on the same
+    hardware can vary across operating systems.
+
+    The returned string may be empty if the CPU architecture couldn't be
+    recognized.
+
+    @see wxGetNativeCpuArchitectureName()
+
+    @since 3.1.5
+*/
+wxString wxGetCpuArchitectureName();
+
+/**
+    In some situations the current process and native CPU architecture may be
+    different. This returns the native CPU architecture regardless of the
+    current process CPU architecture.
+
+    Common examples for CPU architecture differences are the following:
+        - Win32 process in x64 Windows (WoW)
+        - Win32 or x64 process on ARM64 Windows (WoW64)
+        - x86_64 process on ARM64 macOS (Rosetta 2)
+
+    The returned string may be empty if the CPU architecture couldn't be
+    recognized.
+
+    @see wxGetCpuArchitectureName()
+
+    @since 3.1.6
+*/
+wxString wxGetNativeCpuArchitectureName();
+
 /**
     Returns a structure containing information about the currently running
     Linux distribution.
-    
-    This function uses the @c lsb_release utility which is part of the 
-    <tt>Linux Standard Base Core</tt> specification 
-    (see http://refspecs.linux-foundation.org/lsb.shtml) since the very first LSB 
+
+    This function uses the @c lsb_release utility which is part of the
+    <tt>Linux Standard Base Core</tt> specification
+    (see http://refspecs.linux-foundation.org/lsb.shtml) since the very first LSB
     release 1.0 (released in 2001).
     The @c lsb_release utility is very common on modern Linux distributions but in
     case it's not available, then this function will return a ::wxLinuxDistributionInfo
     structure containing empty strings.
-    
+
     This function is Linux-specific and is only available when the @c __LINUX__
     symbol is defined.
 */
@@ -995,8 +1215,7 @@ enum
         Under Unix, if the process is the group leader then passing
         wxKILL_CHILDREN to wxKill() kills all children as well as pid.
 
-        Under MSW, applies only to console applications and is only supported
-        under NT family (i.e. not under Windows 9x). It corresponds to the
+        Under MSW, applies only to console applications. It corresponds to the
         native @c CREATE_NEW_PROCESS_GROUP and, in particular, ensures that
         Ctrl-Break signals will be sent to all children of this process as well
         to the process itself. Support for this flag under MSW was added in
@@ -1033,6 +1252,8 @@ enum
         even if its IO is not redirected.
 
         This flag is ignored under the other platforms.
+
+        @since 2.9.3
      */
     wxEXEC_HIDE_CONSOLE = 32,
 
@@ -1160,10 +1381,10 @@ long wxExecute(const wxString& command, int flags = wxEXEC_ASYNC,
     In wxPerl this function is called @c Wx::ExecuteArgs.
     @endWxPerlOnly
 */
-long wxExecute(char** argv, int flags = wxEXEC_ASYNC,
+long wxExecute(const char* const* argv, int flags = wxEXEC_ASYNC,
                 wxProcess* callback = NULL,
                 const wxExecuteEnv *env = NULL);
-long wxExecute(wchar_t** argv, int flags = wxEXEC_ASYNC,
+long wxExecute(const wchar_t* const* argv, int flags = wxEXEC_ASYNC,
                 wxProcess* callback = NULL,
                 const wxExecuteEnv *env = NULL);
 //@}
@@ -1329,7 +1550,7 @@ bool wxShell(const wxString& command = wxEmptyString);
     the @a flags.
 
     @note Note that performing the shutdown requires the corresponding access
-        rights (superuser under Unix, SE_SHUTDOWN privilege under Windows NT)
+        rights (superuser under Unix, SE_SHUTDOWN privilege under Windows)
         and that this function is only implemented under Unix and MSW.
 
     @param flags
@@ -1399,3 +1620,76 @@ void wxUsleep(unsigned long milliseconds);
 
 //@}
 
+
+/** @addtogroup group_funcmacro_misc */
+//@{
+/**
+    Convert decimal integer to 2-character hexadecimal string.
+
+    @param dec
+        A number to be converted.
+    @param buf
+        A pointer to the buffer that receives hexadecimal string (not prefixed
+        by @c 0x). This buffer should be large enough to hold at least
+        3 characters: 2 hexadecimal digits and the terminating null character.
+
+    @remarks
+        Returned string is composed of uppercase hexdecimal characters.
+
+    @header{wx/utils.h}
+*/
+void wxDecToHex(unsigned char dec, wxChar *buf);
+
+/**
+    Convert decimal integer to 2-character hexadecimal string.
+
+    @param dec
+        A number to be converted.
+    @return
+        String containing hexadecimal string, not prefixed by @c 0x,
+        composed of uppercase characters.
+
+    @header{wx/utils.h}
+*/
+wxString wxDecToHex(unsigned char dec);
+
+/**
+    Returns 2 characters of hexadecimal representation of a given number.
+
+    @param dec
+        A number to be converted.
+    @param ch1
+        Pointer to the variable that receives 1st hexadecimal character.
+        It must not be @NULL.
+    @param ch2
+        Pointer to the variable that receives 2nd hexadecimal character.
+        It must not be @NULL.
+
+    @remarks
+        Returned characters are uppercase.
+
+    @header{wx/utils.h}
+*/
+void wxDecToHex(unsigned char dec, char* ch1, char* ch2);
+
+/**
+    Convert 2-character hexadecimal string to decimal integer.
+
+    @param buf
+        String containing uppercase hexadecimal characters, not prefixed
+        by @c 0x. Its length must be at least 2 characters. If it is longer
+        than 2 characters, only first two will be converted to the number.
+
+    @return
+        An integer number between 0 and 255 that is equivalent to the number
+        in @a buf, or @c -1 if @a buf is not a hexadecimal string.
+
+    @header{wx/utils.h}
+*/
+int wxHexToDec(const wxString& buf);
+
+/**
+    @overload
+*/
+int wxHexToDec(const char* buf);
+//@}
