@@ -25,36 +25,28 @@
 
 #ifndef WX_PRECOMP
     #include "wx/frame.h"
+    #include "wx/app.h"     // GetAppDisplayName()
     #include "wx/icon.h"
     #include "wx/log.h"
 #endif
 
 #include "wx/evtloop.h"
+#include "wx/recguard.h"
 #include "wx/sysopt.h"
 
-#include <gtk/gtk.h>
+#include "wx/gtk/private.h"
+#include "wx/gtk/private/gtk3-compat.h"
+#include "wx/gtk/private/stylecontext.h"
+#include "wx/gtk/private/win_gtk.h"
+#include "wx/gtk/private/backend.h"
+
 #ifdef GDK_WINDOWING_X11
     #include <gdk/gdkx.h>
     #include <X11/Xatom.h>  // XA_CARDINAL
     #include "wx/unix/utilsx11.h"
 #endif
-
-#include "wx/gtk/private.h"
-#include "wx/gtk/private/gtk2-compat.h"
-#include "wx/gtk/private/win_gtk.h"
-
-#if wxUSE_LIBHILDON
-    #include <hildon-widgets/hildon-program.h>
-    #include <hildon-widgets/hildon-window.h>
-#endif // wxUSE_LIBHILDON
-
-#if wxUSE_LIBHILDON2
-    #include <hildon/hildon.h>
-#endif // wxUSE_LIBHILDON2
-
-#ifdef __WXGTK3__
-extern GList* wx_sizeEventList;
-void wxGTKSizeRevalidate(wxWindow*);
+#ifdef GDK_WINDOWING_WAYLAND
+    #include <gdk/gdkwayland.h>
 #endif
 
 // ----------------------------------------------------------------------------
@@ -70,6 +62,7 @@ static wxTopLevelWindowGTK *g_activeFrame = NULL;
 
 extern wxCursor g_globalCursor;
 extern wxCursor g_busyCursor;
+extern wxRecursionGuardFlag g_inSizeAllocate;
 
 #ifdef GDK_WINDOWING_X11
 // Whether _NET_REQUEST_FRAME_EXTENTS support is working
@@ -83,19 +76,18 @@ static bool gs_decorCacheValid;
 #ifdef __WXGTK3__
 static bool HasClientDecor(GtkWidget* widget)
 {
-    static bool has;
-    static bool once;
-    if (!once)
-    {
-        once = true;
-        GdkDisplay* display = gtk_widget_get_display(widget);
-        const char* name = g_type_name(G_TYPE_FROM_INSTANCE(display));
-        has =
-            strcmp(name, "GdkWaylandDisplay") == 0 ||
-            strcmp(name, "GdkMirDisplay") == 0 ||
-            strcmp(name, "GdkBroadwayDisplay") == 0;
-    }
-    return has;
+    if (!gtk_window_get_decorated(GTK_WINDOW(widget)))
+        return false;
+
+    GdkDisplay* display = gtk_widget_get_display(widget);
+    if (wxGTKImpl::IsX11(display))
+        return false;
+
+#if defined(GDK_WINDOWING_WAYLAND) && GTK_CHECK_VERSION(3,22,0)
+    if (wxGTKImpl::IsWayland(display) && wx_is_at_least_gtk3(22))
+        return !gdk_wayland_display_prefers_ssd(display);
+#endif
+    return true;
 }
 #else
 static inline bool HasClientDecor(GtkWidget*)
@@ -113,7 +105,7 @@ static void wxgtk_window_set_urgency_hint (GtkWindow *win,
                                            gboolean setting)
 {
 #if GTK_CHECK_VERSION(2,7,0)
-    if (gtk_check_version(2,7,0) == NULL)
+    if (wx_is_at_least_gtk2(7))
         gtk_window_set_urgency_hint(win, setting);
     else
 #endif
@@ -169,6 +161,7 @@ static gboolean gtk_frame_focus_in_callback( GtkWidget *widget,
         default:
             g_source_remove( win->m_urgency_hint );
             // no break, fallthrough to remove hint too
+            wxFALLTHROUGH;
         case -1:
             gtk_window_set_urgency_hint(GTK_WINDOW(widget), false);
             win->m_urgency_hint = -2;
@@ -249,27 +242,30 @@ static void
 size_allocate(GtkWidget*, GtkAllocation* alloc, wxTopLevelWindowGTK* win)
 {
     win->m_useCachedClientSize = true;
+    GtkAllocation a;
+    gtk_widget_get_allocation(win->m_widget, &a);
+    const bool hasClientDecor = HasClientDecor(win->m_widget);
+    if (hasClientDecor)
+    {
+        GtkAllocation a2;
+        gtk_widget_get_allocation(win->m_mainWidget, &a2);
+        wxTopLevelWindowGTK::DecorSize decorSize;
+        decorSize.left = a2.x;
+        decorSize.right = a.width - a2.width - a2.x;
+        decorSize.top = a2.y;
+        decorSize.bottom = a.height - a2.height - a2.y;
+        win->GTKUpdateDecorSize(decorSize);
+    }
     if (win->m_clientWidth  != alloc->width ||
         win->m_clientHeight != alloc->height)
     {
+        wxRecursionGuard setInSizeAllocate(g_inSizeAllocate);
+
         win->m_clientWidth  = alloc->width;
         win->m_clientHeight = alloc->height;
 
-        GtkAllocation a;
-        gtk_widget_get_allocation(win->m_widget, &a);
         wxSize size(a.width, a.height);
-        if (HasClientDecor(win->m_widget))
-        {
-            GtkAllocation a2;
-            gtk_widget_get_allocation(win->m_mainWidget, &a2);
-            wxTopLevelWindowGTK::DecorSize decorSize;
-            decorSize.left = a2.x;
-            decorSize.right = a.width - a2.width - a2.x;
-            decorSize.top = a2.y;
-            decorSize.bottom = a.height - a2.height - a2.y;
-            win->GTKUpdateDecorSize(decorSize);
-        }
-        else
+        if (!hasClientDecor)
         {
             size.x += win->m_decorSize.left + win->m_decorSize.right;
             size.y += win->m_decorSize.top + win->m_decorSize.bottom;
@@ -340,6 +336,8 @@ void wxTopLevelWindowGTK::GTKConfigureEvent(int x, int y)
 
     if (m_x != point.x || m_y != point.y)
     {
+        m_lastPos = wxPoint(m_x, m_y);
+
         m_x = point.x;
         m_y = point.y;
         wxMoveEvent event(point, GetId());
@@ -352,6 +350,16 @@ void wxTopLevelWindowGTK::GTKConfigureEvent(int x, int y)
 // "realize" from m_widget
 //-----------------------------------------------------------------------------
 
+#if GTK_CHECK_VERSION(3,10,0)
+extern "C" {
+static void findTitlebar(GtkWidget* widget, void* data)
+{
+    if (GTK_IS_HEADER_BAR(widget))
+        *static_cast<GtkWidget**>(data) = widget;
+}
+}
+#endif
+
 // we cannot the WM hints and icons before the widget has been realized,
 // so we do this directly after realization
 
@@ -361,6 +369,31 @@ void wxTopLevelWindowGTK::GTKHandleRealized()
 
     GdkWindow* window = gtk_widget_get_window(m_widget);
 
+#if GTK_CHECK_VERSION(3,10,0)
+    if (wx_is_at_least_gtk3(10))
+    {
+        GtkWidget* titlebar = NULL;
+        gtk_container_forall(GTK_CONTAINER(m_widget), findTitlebar, &titlebar);
+        if (titlebar)
+        {
+#if GTK_CHECK_VERSION(3,12,0)
+            if (m_gdkDecor && wx_is_at_least_gtk3(12))
+            {
+                char layout[sizeof("icon,menu:minimize,maximize,close")];
+                snprintf(layout, sizeof(layout), "icon%s:%s%s%s",
+                     m_gdkDecor & GDK_DECOR_MENU ? ",menu" : "",
+                     m_gdkDecor & GDK_DECOR_MINIMIZE ? "minimize," : "",
+                     m_gdkDecor & GDK_DECOR_MAXIMIZE ? "maximize," : "",
+                     m_gdkFunc & GDK_FUNC_CLOSE ? "close" : "");
+                gtk_header_bar_set_decoration_layout(GTK_HEADER_BAR(titlebar), layout);
+            }
+#endif // 3.12
+            // Don't set WM decorations when GTK is using Client Side Decorations
+            m_gdkDecor = 0;
+        }
+    }
+#endif // 3.10
+
     gdk_window_set_decorations(window, (GdkWMDecoration)m_gdkDecor);
     gdk_window_set_functions(window, (GdkWMFunction)m_gdkFunc);
 
@@ -376,6 +409,7 @@ void wxTopLevelWindowGTK::GTKHandleRealized()
         gdk_window_set_cursor(window, cursor);
 
 #ifdef __WXGTK3__
+    wxGCC_WARNING_SUPPRESS(deprecated-declarations)
     if (gtk_window_get_has_resize_grip(GTK_WINDOW(m_widget)))
     {
         // Grip window can end up obscured, probably due to deferred show.
@@ -383,6 +417,7 @@ void wxTopLevelWindowGTK::GTKHandleRealized()
         gtk_window_set_has_resize_grip(GTK_WINDOW(m_widget), false);
         gtk_window_set_has_resize_grip(GTK_WINDOW(m_widget), true);
     }
+    wxGCC_WARNING_RESTORE()
 #endif
 }
 
@@ -412,9 +447,7 @@ gtk_frame_map_callback( GtkWidget*,
     // it is possible for m_isShown to be false here, see bug #9909
     if (win->wxWindowBase::Show(true))
     {
-        wxShowEvent eventShow(win->GetId(), true);
-        eventShow.SetEventObject(win);
-        win->GetEventHandler()->ProcessEvent(eventShow);
+        win->GTKDoAfterShow();
     }
 
     // restore focus-on-map setting in case ShowWithoutActivating() was called
@@ -472,8 +505,10 @@ bool wxGetFrameExtents(GdkWindow* window, int* left, int* right, int* top, int*
 #ifdef GDK_WINDOWING_X11
     GdkDisplay* display = gdk_window_get_display(window);
 
-    if (!GDK_IS_X11_DISPLAY(display))
+#ifdef __WXGTK3__
+    if (!wxGTKImpl::IsX11(display))
         return false;
+#endif
 
     static GdkAtom property = gdk_atom_intern("_NET_FRAME_EXTENTS", false);
     Atom xproperty = gdk_x11_atom_to_xatom_for_display(display, property);
@@ -493,7 +528,7 @@ bool wxGetFrameExtents(GdkWindow* window, int* left, int* right, int* top, int*
         // We need to convert the X11 physical extents to GTK+ "logical" units
         int scale = 1;
 #if GTK_CHECK_VERSION(3,10,0)
-        if (gtk_check_version(3,10,0) == NULL)
+        if (wx_is_at_least_gtk3(10))
             scale = gdk_window_get_scale_factor(window);
 #endif
         long* p = (long*)data;
@@ -571,15 +606,18 @@ void wxTopLevelWindowGTK::Init()
     m_themeEnabled = true;
     m_gdkDecor =
     m_gdkFunc = 0;
-    m_grabbed = false;
+    m_grabbedEventLoop = NULL;
     m_deferShow = true;
     m_deferShowAllowed = true;
     m_updateDecorSize = true;
     m_netFrameExtentsTimerId = 0;
     m_incWidth = m_incHeight = 0;
-    memset(&m_decorSize, 0, sizeof(m_decorSize));
 
     m_urgency_hint = -2;
+
+#ifdef __WXGTK3__
+    m_pendingFittingClientSizeFlags = 0;
+#endif // __WXGTK3__
 }
 
 bool wxTopLevelWindowGTK::Create( wxWindow *parent,
@@ -605,18 +643,15 @@ bool wxTopLevelWindowGTK::Create( wxWindow *parent,
 
     m_title = title;
 
+#ifndef __WXGTK4__
+    // Gnome uses class as display name
+    gdk_set_program_class(wxTheApp->GetAppDisplayName().utf8_str());
+#endif
+
     // NB: m_widget may be !=NULL if it was created by derived class' Create,
     //     e.g. in wxTaskBarIconAreaGTK
     if (m_widget == NULL)
     {
-#if wxUSE_LIBHILDON || wxUSE_LIBHILDON2
-        // we must create HildonWindow and not a normal GtkWindow as the latter
-        // doesn't look correctly in Maemo environment and it must also be
-        // registered with the main program object
-        m_widget = hildon_window_new();
-        hildon_program_add_window(wxTheApp->GetHildonProgram(),
-                                  HILDON_WINDOW(m_widget));
-#else // !wxUSE_LIBHILDON || !wxUSE_LIBHILDON2
         m_widget = gtk_window_new(GTK_WINDOW_TOPLEVEL);
         if (GetExtraStyle() & wxTOPLEVEL_EX_DIALOG)
         {
@@ -643,7 +678,6 @@ bool wxTopLevelWindowGTK::Create( wxWindow *parent,
                 style |= wxFRAME_NO_TASKBAR;
             }
         }
-#endif // wxUSE_LIBHILDON || wxUSE_LIBHILDON2/!wxUSE_LIBHILDON || !wxUSE_LIBHILDON2
 
         g_object_ref(m_widget);
     }
@@ -738,9 +772,12 @@ bool wxTopLevelWindowGTK::Create( wxWindow *parent,
     g_signal_connect (m_widget, "key_press_event",
                       G_CALLBACK (wxgtk_tlw_key_press_event), NULL);
 
+#ifdef __WXGTK3__
+    GdkDisplay* display = gtk_widget_get_display(m_widget);
+#endif
 #ifdef GDK_WINDOWING_X11
 #ifdef __WXGTK3__
-    if (GDK_IS_X11_SCREEN(gtk_window_get_screen(GTK_WINDOW(m_widget))))
+    if (wxGTKImpl::IsX11(display))
 #endif
     {
         gtk_widget_add_events(m_widget, GDK_PROPERTY_CHANGE_MASK);
@@ -778,7 +815,7 @@ bool wxTopLevelWindowGTK::Create( wxWindow *parent,
             m_gdkDecor |= GDK_DECOR_TITLE;
 #if GTK_CHECK_VERSION(3,10,0)
         else if (
-            strcmp("GdkWaylandDisplay", g_type_name(G_TYPE_FROM_INSTANCE(gtk_widget_get_display(m_widget)))) == 0 &&
+            wxGTKImpl::IsWayland(display) &&
             gtk_check_version(3,10,0) == NULL)
         {
             gtk_window_set_titlebar(GTK_WINDOW(m_widget), gtk_header_bar_new());
@@ -802,11 +839,12 @@ bool wxTopLevelWindowGTK::Create( wxWindow *parent,
     }
 
     m_decorSize = GetCachedDecorSize();
-    int w, h;
-    GTKDoGetSize(&w, &h);
+    int w = m_width;
+    int h = m_height;
 
     if (style & wxRESIZE_BORDER)
     {
+        GTKDoGetSize(&w, &h);
         gtk_window_set_default_size(GTK_WINDOW(m_widget), w, h);
 #ifndef __WXGTK3__
         gtk_window_set_policy(GTK_WINDOW(m_widget), 1, 1, 1);
@@ -818,10 +856,18 @@ bool wxTopLevelWindowGTK::Create( wxWindow *parent,
         // gtk_window_set_default_size() does not work for un-resizable windows,
         // unless you set the size hints, but that causes Ubuntu's WM to make
         // the window resizable even though GDK_FUNC_RESIZE is not set.
+        if (!HasClientDecor(m_widget))
+            GTKDoGetSize(&w, &h);
         gtk_widget_set_size_request(m_widget, w, h);
     }
 
-    g_signal_connect(gtk_settings_get_default(), "notify::gtk-theme-name",
+    // Note that we need to connect after this signal in order to let the
+    // normal g_signal_connect() in wxSystemSettings code to run before our
+    // handler: this ensures that system settings cache is cleared before the
+    // user-defined wxSysColourChangedEvent handlers using wxSystemSettings
+    // methods are executed, which is important as otherwise they would use the
+    // old colours etc.
+    g_signal_connect_after(gtk_settings_get_default(), "notify::gtk-theme-name",
         G_CALLBACK(notify_gtk_theme_name), this);
 
     return true;
@@ -836,16 +882,7 @@ wxTopLevelWindowGTK::~wxTopLevelWindowGTK()
         g_source_remove(m_netFrameExtentsTimerId);
     }
 
-#if wxUSE_LIBHILDON || wxUSE_LIBHILDON2
-    // it can also be a (standard) dialog
-    if ( HILDON_IS_WINDOW(m_widget) )
-    {
-        hildon_program_remove_window(wxTheApp->GetHildonProgram(),
-                                     HILDON_WINDOW(m_widget));
-    }
-#endif // wxUSE_LIBHILDON || wxUSE_LIBHILDON2
-
-    if (m_grabbed)
+    if (m_grabbedEventLoop)
     {
         wxFAIL_MSG(wxT("Window still grabbed"));
         RemoveGrab();
@@ -885,16 +922,23 @@ bool wxTopLevelWindowGTK::ShowFullScreen(bool show, long)
     if (show == m_fsIsShowing)
         return false; // return what?
 
+    // documented behaviour is to show the window if it's still hidden when
+    // showing it full screen
+    if (show)
+        Show();
+
     m_fsIsShowing = show;
 
-#ifdef GDK_WINDOWING_X11
+#if defined(GDK_WINDOWING_X11) && !defined(__WXGTK4__)
     GdkScreen* screen = gtk_widget_get_screen(m_widget);
     GdkDisplay* display = gdk_screen_get_display(screen);
     Display* xdpy = NULL;
     Window xroot = None;
     wxX11FullScreenMethod method = wxX11_FS_WMSPEC;
 
-    if (GDK_IS_X11_DISPLAY(display))
+#ifdef __WXGTK3__
+    if (wxGTKImpl::IsX11(display))
+#endif
     {
         xdpy = GDK_DISPLAY_XDISPLAY(display);
         xroot = GDK_WINDOW_XID(gdk_screen_get_root_window(screen));
@@ -913,7 +957,7 @@ bool wxTopLevelWindowGTK::ShowFullScreen(bool show, long)
         else
             gtk_window_unfullscreen( GTK_WINDOW( m_widget ) );
     }
-#ifdef GDK_WINDOWING_X11
+#if defined(GDK_WINDOWING_X11) && !defined(__WXGTK4__)
     else if (xdpy != NULL)
     {
         GdkWindow* window = gtk_widget_get_window(m_widget);
@@ -924,8 +968,10 @@ bool wxTopLevelWindowGTK::ShowFullScreen(bool show, long)
             GetPosition( &m_fsSaveFrame.x, &m_fsSaveFrame.y );
             GetSize( &m_fsSaveFrame.width, &m_fsSaveFrame.height );
 
+            wxGCC_WARNING_SUPPRESS(deprecated-declarations)
             const int screen_width = gdk_screen_get_width(screen);
             const int screen_height = gdk_screen_get_height(screen);
+            wxGCC_WARNING_RESTORE()
 
             gint client_x, client_y, root_x, root_y;
             gint width, height;
@@ -965,11 +1011,6 @@ bool wxTopLevelWindowGTK::ShowFullScreen(bool show, long)
     }
 #endif // GDK_WINDOWING_X11
 
-    // documented behaviour is to show the window if it's still hidden when
-    // showing it full screen
-    if (show)
-        Show();
-
     return true;
 }
 
@@ -990,14 +1031,38 @@ void wxTopLevelWindowGTK::Refresh( bool WXUNUSED(eraseBackground), const wxRect
         gdk_window_invalidate_rect(window, NULL, true);
 }
 
+#if defined(__WXGTK3__) && defined(GDK_WINDOWING_X11)
+// Check conditions under which GTK will use Client Side Decorations with X11
+static bool isUsingCSD(GtkWidget* widget)
+{
+    const char* csd = getenv("GTK_CSD");
+    if (csd == NULL || strcmp(csd, "1") != 0)
+        return false;
+
+    GdkScreen* screen = gtk_widget_get_screen(widget);
+    if (!gdk_screen_is_composited(screen))
+        return false;
+
+    GdkAtom atom = gdk_atom_intern_static_string("_GTK_FRAME_EXTENTS");
+    if (!gdk_x11_screen_supports_net_wm_hint(screen, atom))
+        return false;
+
+    if (gdk_screen_get_rgba_visual(screen) == NULL)
+        return false;
+
+    return true;
+}
+#endif // __WXGTK3__ && GDK_WINDOWING_X11
+
 bool wxTopLevelWindowGTK::Show( bool show )
 {
     wxCHECK_MSG(m_widget, false, "invalid frame");
 
 #ifdef GDK_WINDOWING_X11
-    bool deferShow = show && !m_isShown && m_deferShow;
+    bool deferShow = show && !m_isShown && !m_isIconized && m_deferShow;
     if (deferShow)
     {
+        GdkScreen* screen = gtk_widget_get_screen(m_widget);
         deferShow = m_deferShowAllowed &&
             // Assume size (from cache or wxPersistentTLW) is correct.
             // Avoids problems when WM initially provides an incorrect value
@@ -1006,14 +1071,20 @@ bool wxTopLevelWindowGTK::Show( bool show )
 
             gs_requestFrameExtentsStatus != RFE_STATUS_BROKEN &&
             !gtk_widget_get_realized(m_widget) &&
-            GDK_IS_X11_DISPLAY(gtk_widget_get_display(m_widget)) &&
+#ifdef __WXGTK3__
+            wxGTKImpl::IsX11(screen) &&
+#endif
             g_signal_handler_find(m_widget,
                 GSignalMatchType(G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_DATA),
                 g_signal_lookup("property_notify_event", GTK_TYPE_WIDGET),
                 0, NULL, NULL, this);
+#ifdef __WXGTK3__
+        // Don't defer with CSD, it isn't needed and causes pixman errors
+        if (deferShow)
+            deferShow = !isUsingCSD(m_widget);
+#endif
         if (deferShow)
         {
-            GdkScreen* screen = gtk_widget_get_screen(m_widget);
             GdkAtom atom = gdk_atom_intern("_NET_REQUEST_FRAME_EXTENTS", false);
             deferShow = gdk_x11_screen_supports_net_wm_hint(screen, atom) != 0;
 
@@ -1091,22 +1162,34 @@ bool wxTopLevelWindowGTK::Show( bool show )
         SendSizeEvent();
 
 #ifdef __WXGTK3__
-        wxGTKSizeRevalidate(this);
+        GTKSizeRevalidate();
 #endif
     }
 
     bool change = base_type::Show(show);
 
 #ifdef __WXGTK3__
-    if (GList* p = g_list_find(wx_sizeEventList, this))
+    if (change && show)
     {
-        wx_sizeEventList = g_list_delete_link(wx_sizeEventList, p);
-        SendSizeEvent();
+        // We may need to redo it after showing the window.
+        GTKUpdateClientSizeIfNecessary();
     }
+
+    GTKSendSizeEventIfNeeded();
 #endif
 
     if (change && !show)
     {
+        // Generate wxEVT_KILL_FOCUS for the currently focused control
+        // immediately (i.e. without waiting until the window is destroyed and
+        // doing it from its dtor), as it could be too late to execute the
+        // handler for this event, or other events triggered by receiving it,
+        // by then because the wxTLW will have been half-destroyed by then.
+        if (GTK_IS_WINDOW(m_widget))
+        {
+            gtk_window_set_focus( GTK_WINDOW(m_widget), NULL );
+        }
+
         // make sure window has a non-default position, so when it is shown
         // again, it won't be repositioned by WM as if it were a new window
         // Note that this must be done _after_ the window is hidden.
@@ -1142,32 +1225,12 @@ void wxTopLevelWindowGTK::DoMoveWindow(int WXUNUSED(x), int WXUNUSED(y), int WXU
 void wxTopLevelWindowGTK::GTKDoGetSize(int *width, int *height) const
 {
     wxSize size(m_width, m_height);
-    if (!HasClientDecor(m_widget))
-    {
-        size.x -= m_decorSize.left + m_decorSize.right;
-        size.y -= m_decorSize.top + m_decorSize.bottom;
-    }
+    size.x -= m_decorSize.left + m_decorSize.right;
+    size.y -= m_decorSize.top + m_decorSize.bottom;
     if (size.x < 0) size.x = 0;
     if (size.y < 0) size.y = 0;
-#if wxUSE_LIBHILDON2
-    if (width) {
-       if (size.x == 720)
-               *width = 696;
-       else
-               *width = size.x;
-    }
-    if (height) {
-       if (size.y == 420)
-               *height = 396;
-       else if (size.y == 270)
-               *height = 246;
-            else
-               *height = size.y;
-    }
-#else // wxUSE_LIBHILDON2
     if (width)  *width  = size.x;
     if (height) *height = size.y;
-#endif // wxUSE_LIBHILDON2 /!wxUSE_LIBHILDON2
 }
 
 void wxTopLevelWindowGTK::DoSetSize( int x, int y, int width, int height, int sizeFlags )
@@ -1214,11 +1277,32 @@ void wxTopLevelWindowGTK::DoSetSize( int x, int y, int width, int height, int si
         m_deferShowAllowed = true;
         m_useCachedClientSize = false;
 
-        int w, h;
-        GTKDoGetSize(&w, &h);
-        gtk_window_resize(GTK_WINDOW(m_widget), w, h);
-        if (!gtk_window_get_resizable(GTK_WINDOW(m_widget)))
+#ifdef __WXGTK3__
+        // Reset pending client size, it is not relevant any more and shouldn't
+        // be set when the window is shown, as we don't want it to replace the
+        // size explicitly specified here. Note that we do still want to set
+        // the minimum client size, as increasing the total size shouldn't
+        // allow shrinking the frame beyond its minimum fitting size.
+        //
+        // Also note that if we're called from WXSetInitialFittingClientSize()
+        // itself, this will be overwritten again with the pending flags when
+        // we return.
+        m_pendingFittingClientSizeFlags &= ~wxSIZE_SET_CURRENT;
+#endif // __WXGTK3__
+
+        int w = m_width;
+        int h = m_height;
+        if (gtk_window_get_resizable(GTK_WINDOW(m_widget)))
+        {
+            GTKDoGetSize(&w, &h);
+            gtk_window_resize(GTK_WINDOW(m_widget), w, h);
+        }
+        else
+        {
+            if (!HasClientDecor(m_widget))
+                GTKDoGetSize(&w, &h);
             gtk_widget_set_size_request(GTK_WIDGET(m_widget), w, h);
+        }
 
         DoGetClientSize(&m_clientWidth, &m_clientHeight);
         wxSizeEvent event(GetSize(), GetId());
@@ -1297,6 +1381,17 @@ void wxTopLevelWindowGTK::DoSetSizeHints( int minW, int minH,
                                           int incW, int incH )
 {
     base_type::DoSetSizeHints(minW, minH, maxW, maxH, incW, incH);
+
+    if (!HasFlag(wxRESIZE_BORDER))
+    {
+        // It's not useful to set size hints for the windows which can't be
+        // resized anyhow, and it even results in warnings from Gnome window
+        // manager, so just don't do it (but notice that it's not an error to
+        // call this for non-resizeable windows, e.g. because it's done
+        // implicitly by SetSizerAndFit() which is useful even in this case).
+        return;
+    }
+
     m_incWidth = incW;
     m_incHeight = incH;
 
@@ -1416,7 +1511,7 @@ void wxTopLevelWindowGTK::GTKUpdateDecorSize(const DecorSize& decorSize)
         SendSizeEvent();
 
 #ifdef __WXGTK3__
-        wxGTKSizeRevalidate(this);
+        GTKSizeRevalidate();
 #endif
         if (!m_isShown)
             return;
@@ -1424,15 +1519,9 @@ void wxTopLevelWindowGTK::GTKUpdateDecorSize(const DecorSize& decorSize)
         gtk_widget_show(m_widget);
 
 #ifdef __WXGTK3__
-        if (GList* p = g_list_find(wx_sizeEventList, this))
-        {
-            wx_sizeEventList = g_list_delete_link(wx_sizeEventList, p);
-            SendSizeEvent();
-        }
+        GTKSendSizeEventIfNeeded();
 #endif
-        wxShowEvent showEvent(GetId(), true);
-        showEvent.SetEventObject(this);
-        HandleWindowEvent(showEvent);
+        GTKDoAfterShow();
     }
 #endif // GDK_WINDOWING_X11
 }
@@ -1454,11 +1543,55 @@ wxTopLevelWindowGTK::DecorSize& wxTopLevelWindowGTK::GetCachedDecorSize()
     return size[index];
 }
 
-void wxTopLevelWindowGTK::OnInternalIdle()
+void wxTopLevelWindowGTK::GTKDoAfterShow()
 {
-    wxTopLevelWindowBase::OnInternalIdle();
+#ifdef __WXGTK3__
+    // Set the client size again if necessary, we should be able to do it
+    // correctly by now as the style cache should be up to date.
+    GTKUpdateClientSizeIfNecessary();
+#endif // __WXGTK3__
+
+    wxShowEvent showEvent(GetId(), true);
+    showEvent.SetEventObject(this);
+    HandleWindowEvent(showEvent);
 }
 
+#ifdef __WXGTK3__
+
+void wxTopLevelWindowGTK::GTKUpdateClientSizeIfNecessary()
+{
+    if ( m_pendingFittingClientSizeFlags )
+    {
+        WXSetInitialFittingClientSize(m_pendingFittingClientSizeFlags);
+
+        m_pendingFittingClientSizeFlags = 0;
+    }
+}
+
+void wxTopLevelWindowGTK::SetMinSize(const wxSize& minSize)
+{
+    wxTopLevelWindowBase::SetMinSize(minSize);
+
+    // Explicitly set minimum size should override the pending size, if any.
+    m_pendingFittingClientSizeFlags &= ~wxSIZE_SET_MIN;
+}
+
+void
+wxTopLevelWindowGTK::WXSetInitialFittingClientSize(int flags, wxSizer* sizer)
+{
+    // In any case, update the size immediately.
+    wxTopLevelWindowBase::WXSetInitialFittingClientSize(flags, sizer);
+
+    // But if we're not shown yet, the fitting size may be wrong because GTK
+    // style cache hasn't been updated yet and we need to do it again when the
+    // window becomes visible as we can be sure that by then we'll be able to
+    // compute the best size correctly.
+    if ( !IsShown() )
+        m_pendingFittingClientSizeFlags = flags;
+}
+
+#endif // __WXGTK3__
+
 // ----------------------------------------------------------------------------
 // frame title/icon
 // ----------------------------------------------------------------------------
@@ -1480,7 +1613,7 @@ void wxTopLevelWindowGTK::SetIcons( const wxIconBundle &icons )
     base_type::SetIcons(icons);
 
     // Setting icons before window is realized can cause a GTK assertion if
-    // another TLW is realized before this one, and it has this one as it's
+    // another TLW is realized before this one, and it has this one as its
     // transient parent. The life demo exibits this problem.
     if (m_widget && gtk_widget_get_realized(m_widget))
     {
@@ -1535,6 +1668,62 @@ void wxTopLevelWindowGTK::SetIconizeState(bool iconize)
 {
     if ( iconize != m_isIconized )
     {
+        /*
+            We get a configure-event _before_ the window is iconized with dummy
+            (0, 0) coordinates. Unfortunately we can't just ignore this event
+            when we get it, because we can't know if we're going to be iconized
+            or not: even remembering that we should be soon in Iconize() itself
+            doesn't help due to the crazy sequence of events generated by GTK,
+            e.g. under X11 for the sequence of Iconize() and Show() calls we
+            get the following events with GTK2:
+
+            - window-state changes to GDK_WINDOW_STATE_ICONIFIED | WITHDRAWN
+            - window-state changes to just GDK_WINDOW_STATE_ICONIFIED
+            - map event
+            - window-state changes to normal
+            - configure event with normal size
+            - window-state changes to GDK_WINDOW_STATE_ICONIFIED
+            - configure event with normal size
+            - configure event with (0, 0) size
+            - window-state changes to normal (yes, again)
+            - configure event with (0, 0) size
+            - window-state changes to GDK_WINDOW_STATE_ICONIFIED
+
+            So even though we could ignore the first (0, 0) configure event, we
+            still wouldn't be able to ignore the second one, happening after
+            the inexplicable switch to normal state.
+
+            For completeness, with GTK3 the sequence of events is simpler, but
+            still very unhelpful for our purposes:
+
+            - window-state changes to GDK_WINDOW_STATE_ICONIFIED
+            - window-state changes to normal
+            - map event
+            - configure event with normal size
+            - configure event with normal size (yes, because one is not enough)
+            - configure event with (0, 0) size
+            - configure event with (0, 0) size (because why not have 2 of them)
+            - window-state changes to GDK_WINDOW_STATE_ICONIFIED
+
+            Here again we have (0, 0) configure events happening _after_ the
+            window-state switch to normal, that would reset our flag.
+
+            In conclusion, we have no choice but to assume that the window got
+            really moved, but at least we can remember its previous position in
+            order to restore it here if it turns out that it was just iconized.
+         */
+        if ( iconize )
+        {
+            if ( wxPoint(m_x, m_y) == wxPoint(0, 0) )
+            {
+                m_x = m_lastPos.x;
+                m_y = m_lastPos.y;
+
+                // This is not really necessary, but seems tidier.
+                m_lastPos = wxPoint();
+            }
+        }
+
         m_isIconized = iconize;
         (void)SendIconizeEvent(iconize);
     }
@@ -1542,24 +1731,31 @@ void wxTopLevelWindowGTK::SetIconizeState(bool iconize)
 
 void wxTopLevelWindowGTK::AddGrab()
 {
-    if (!m_grabbed)
+    if (!m_grabbedEventLoop)
     {
-        m_grabbed = true;
+        wxGUIEventLoop eventLoop;
+        m_grabbedEventLoop = &eventLoop;
         gtk_grab_add( m_widget );
-        wxGUIEventLoop().Run();
+        eventLoop.Run();
         gtk_grab_remove( m_widget );
+        m_grabbedEventLoop = NULL;
     }
 }
 
 void wxTopLevelWindowGTK::RemoveGrab()
 {
-    if (m_grabbed)
+    if (m_grabbedEventLoop)
     {
-        wxGUIEventLoop::GetActive()->Exit();
-        m_grabbed = false;
+        m_grabbedEventLoop->Exit();
+        m_grabbedEventLoop = NULL;
     }
 }
 
+bool wxTopLevelWindowGTK::IsGrabbed() const
+{
+    return m_grabbedEventLoop != NULL;
+}
+
 bool wxTopLevelWindowGTK::IsActive()
 {
     return (this == (wxTopLevelWindowGTK*)g_activeFrame);
@@ -1624,18 +1820,47 @@ void wxTopLevelWindowGTK::SetWindowStyleFlag( long style )
 
 bool wxTopLevelWindowGTK::SetTransparent(wxByte alpha)
 {
-    if (m_widget == NULL)
+    wxCHECK_MSG(m_widget, false, "invalid window");
+
+#ifdef __WXGTK3__
+    // RGBA visual is required
+    GdkScreen* screen = gtk_widget_get_screen(m_widget);
+    GdkVisual* visual = gdk_screen_get_rgba_visual(screen);
+    if (visual == NULL)
         return false;
-#if GTK_CHECK_VERSION(2,12,0)
-#ifndef __WXGTK3__
-    if (gtk_check_version(2,12,0) == NULL)
+    if (gtk_widget_get_visual(m_widget) != visual)
+    {
+        if (gtk_widget_get_realized(m_widget))
+        {
+            wxLogDebug("SetTransparent() must be called before Show()");
+        }
+        gtk_widget_set_visual(m_widget, visual);
+    }
+#if GTK_CHECK_VERSION(3,8,0)
+    if (wx_is_at_least_gtk3(8))
+    {
+        gtk_widget_set_opacity(m_widget, alpha / 255.0);
+    }
+    else
+#endif // GTK+ 3.8+
+    {
+#ifndef __WXGTK4__
+        wxGCC_WARNING_SUPPRESS(deprecated-declarations)
+        gtk_window_set_opacity(GTK_WINDOW(m_widget), alpha / 255.0);
+        wxGCC_WARNING_RESTORE()
 #endif
+    }
+
+    return true;
+#else // !__WXGTK3__
+
+#if GTK_CHECK_VERSION(2,12,0)
+    if (wx_is_at_least_gtk2(12))
     {
         gtk_window_set_opacity(GTK_WINDOW(m_widget), alpha / 255.0);
         return true;
     }
 #endif // GTK_CHECK_VERSION(2,12,0)
-#ifndef __WXGTK3__
 #ifdef GDK_WINDOWING_X11
     GdkWindow* window = gtk_widget_get_window(m_widget);
     if (window == NULL)
@@ -1658,7 +1883,7 @@ bool wxTopLevelWindowGTK::SetTransparent(wxByte alpha)
 #else // !GDK_WINDOWING_X11
     return false;
 #endif // GDK_WINDOWING_X11 / !GDK_WINDOWING_X11
-#endif // !__WXGTK3__
+#endif // __WXGTK3__/!__WXGTK3__
 }
 
 bool wxTopLevelWindowGTK::CanSetTransparent()
@@ -1670,20 +1895,22 @@ bool wxTopLevelWindowGTK::CanSetTransparent()
         return wxSystemOptions::GetOptionInt(SYSOPT_TRANSPARENT) != 0;
     }
 
-#ifdef __WXGTK3__
-    return gtk_widget_is_composited(m_widget) != 0;
+#ifdef __WXGTK4__
+    return gdk_display_is_composited(gtk_widget_get_display(m_widget)) != 0;
 #else
 #if GTK_CHECK_VERSION(2,10,0)
-    if (!gtk_check_version(2,10,0))
+    if (wx_is_at_least_gtk2(10))
     {
+        wxGCC_WARNING_SUPPRESS(deprecated-declarations)
         return gtk_widget_is_composited(m_widget) != 0;
+        wxGCC_WARNING_RESTORE()
     }
     else
 #endif // In case of lower versions than gtk+-2.10.0 we could look for _NET_WM_CM_Sn ourselves
     {
         return false;
     }
-#endif // !__WXGTK3__
+#endif
 
 #if 0 // Don't be optimistic here for the sake of wxAUI
     int opcode, event, error;
@@ -1692,3 +1919,12 @@ bool wxTopLevelWindowGTK::CanSetTransparent()
                            "Composite", &opcode, &event, &error);
 #endif
 }
+
+wxVisualAttributes wxTopLevelWindowGTK::GetDefaultAttributes() const
+{
+    wxVisualAttributes attrs(GetClassDefaultAttributes());
+#ifdef __WXGTK3__
+    wxGtkStyleContext().AddWindow().Bg(attrs.colBg);
+#endif
+    return attrs;
+}
