@@ -58,13 +58,7 @@ class wxFontRefData : public wxGDIRefData
 {
 public:
     // from broken down font parameters, also default ctor
-    wxFontRefData(int size = -1,
-                  wxFontFamily family = wxFONTFAMILY_DEFAULT,
-                  wxFontStyle style = wxFONTSTYLE_NORMAL,
-                  wxFontWeight weight = wxFONTWEIGHT_NORMAL,
-                  bool underlined = false,
-                  const wxString& faceName = wxEmptyString,
-                  wxFontEncoding encoding = wxFONTENCODING_DEFAULT);
+    wxFontRefData(const wxFontInfo& info = wxFontInfo());
 
     // from XFLD
     wxFontRefData(const wxString& fontname);
@@ -83,10 +77,10 @@ public:
 
     // setters: all of them also take care to modify m_nativeFontInfo if we
     // have it so as to not lose the information not carried by our fields
-    void SetPointSize(int pointSize);
+    void SetFractionalPointSize(double pointSize);
     void SetFamily(wxFontFamily family);
     void SetStyle(wxFontStyle style);
-    void SetWeight(wxFontWeight weight);
+    void SetNumericWeight(int weight);
     void SetUnderlined(bool underlined);
     bool SetFaceName(const wxString& facename);
     void SetEncoding(wxFontEncoding encoding);
@@ -96,10 +90,10 @@ public:
 
 protected:
     // common part of all ctors
-    void Init(int pointSize,
+    void Init(float pointSize,
               wxFontFamily family,
               wxFontStyle style,
-              wxFontWeight weight,
+              int weight,
               bool underlined,
               const wxString& faceName,
               wxFontEncoding encoding);
@@ -114,10 +108,10 @@ private:
     // the map of font sizes to "GdkFont *"
     wxScaledFontList  m_scaled_xfonts;
 
-    int             m_pointSize;
+    double          m_pointSize;
     wxFontFamily    m_family;
     wxFontStyle     m_style;
-    wxFontWeight    m_weight;
+    int             m_weight;
     bool            m_underlined;
     wxString        m_faceName;
     wxFontEncoding  m_encoding;  // Unused under GTK 2.0
@@ -135,10 +129,10 @@ private:
 // wxFontRefData
 // ----------------------------------------------------------------------------
 
-void wxFontRefData::Init(int pointSize,
+void wxFontRefData::Init(float pointSize,
                          wxFontFamily family,
                          wxFontStyle style,
-                         wxFontWeight weight,
+                         int weight,
                          bool underlined,
                          const wxString& faceName,
                          wxFontEncoding encoding)
@@ -147,15 +141,10 @@ void wxFontRefData::Init(int pointSize,
 
     m_faceName = faceName;
 
-    // we accept both wxDEFAULT and wxNORMAL here - should we?
-    m_style = style == wxDEFAULT ? wxFONTSTYLE_NORMAL : style;
-    m_weight = weight == wxDEFAULT ? wxFONTWEIGHT_NORMAL : weight;
+    m_style = style;
+    m_weight = weight;
 
-    // and here, do we really want to forbid creation of the font of the size
-    // 90 (the value of wxDEFAULT)??
-    m_pointSize = pointSize == wxDEFAULT || pointSize == -1
-                    ? wxDEFAULT_FONT_SIZE
-                    : pointSize;
+    m_pointSize = pointSize < 0 ? wxDEFAULT_FONT_SIZE : pointSize;
 
     m_underlined = underlined;
     m_encoding = encoding;
@@ -206,14 +195,14 @@ void wxFontRefData::InitFromNative()
     if ( m_nativeFontInfo.GetXFontComponent(wxXLFD_POINTSIZE).ToLong(&ptSize) )
     {
         // size in XLFD is in 10 point units
-        m_pointSize = (int)(ptSize / 10);
+        m_pointSize = ptSize / 10;
     }
     else
     {
         m_pointSize = wxDEFAULT_FONT_SIZE;
     }
 
-    // examine the spacing: if the font is monospaced, assume wxTELETYPE
+    // examine the spacing: if the font is monospaced, assume wxFONTFAMILY_TELETYPE
     // family for compatibility with the old code which used it instead of
     // IsFixedWidth()
     if ( m_nativeFontInfo.GetXFontComponent(wxXLFD_SPACING).Upper() == wxT('M') )
@@ -281,12 +270,15 @@ wxFontRefData::wxFontRefData( const wxFontRefData& data )
     m_nativeFontInfo.FromString(data.m_nativeFontInfo.ToString());
 }
 
-wxFontRefData::wxFontRefData(int size, wxFontFamily family, wxFontStyle style,
-                             wxFontWeight weight, bool underlined,
-                             const wxString& faceName,
-                             wxFontEncoding encoding)
+wxFontRefData::wxFontRefData(const wxFontInfo& info)
 {
-    Init(size, family, style, weight, underlined, faceName, encoding);
+    Init(info.GetFractionalPointSize(),
+         info.GetFamily(),
+         info.GetStyle(),
+         info.GetNumericWeight(),
+         info.IsUnderlined(),
+         info.GetFaceName(),
+         info.GetEncoding());
 }
 
 wxFontRefData::wxFontRefData(const wxString& fontname)
@@ -319,19 +311,13 @@ wxFontRefData::~wxFontRefData()
 // wxFontRefData SetXXX()
 // ----------------------------------------------------------------------------
 
-void wxFontRefData::SetPointSize(int pointSize)
+void wxFontRefData::SetFractionalPointSize(double pointSize)
 {
     m_pointSize = pointSize;
 
     if ( HasNativeFont() )
     {
-        wxString size;
-        if ( pointSize == -1 )
-            size = wxT('*');
-        else
-            size.Printf(wxT("%d"), 10*pointSize);
-
-        m_nativeFontInfo.SetXFontComponent(wxXLFD_POINTSIZE, size);
+        m_nativeFontInfo.SetFractionalPointSize(pointSize);
     }
 }
 
@@ -371,33 +357,13 @@ void wxFontRefData::SetStyle(wxFontStyle style)
     }
 }
 
-void wxFontRefData::SetWeight(wxFontWeight weight)
+void wxFontRefData::SetNumericWeight(int weight)
 {
     m_weight = weight;
 
     if ( HasNativeFont() )
     {
-        wxString boldness;
-        switch ( weight )
-        {
-            case wxFONTWEIGHT_BOLD:
-                boldness = wxT("bold");
-                break;
-
-            case wxFONTWEIGHT_LIGHT:
-                boldness = wxT("light");
-                break;
-
-            default:
-                wxFAIL_MSG( wxT("unknown font weight") );
-                // fall through
-
-            case wxFONTWEIGHT_NORMAL:
-                // unspecified
-                boldness = wxT("medium");
-        }
-
-        m_nativeFontInfo.SetXFontComponent(wxXLFD_WEIGHT, boldness);
+        m_nativeFontInfo.SetNumericWeight(weight);
     }
 }
 
@@ -465,8 +431,9 @@ bool wxFont::Create( int pointSize,
 {
     UnRef();
 
-    m_refData = new wxFontRefData(pointSize, family, style, weight,
-                                  underlined, face, encoding);
+    m_refData = new wxFontRefData(InfoFromLegacyParams(pointSize, family,
+                                                       style, weight, underlined,
+                                                       face, encoding));
 
     return true;
 }
@@ -518,7 +485,7 @@ wxGDIRefData *wxFont::CloneGDIRefData(const wxGDIRefData *data) const
 // accessors
 // ----------------------------------------------------------------------------
 
-int wxFont::GetPointSize() const
+double wxFont::GetFractionalPointSize() const
 {
     wxCHECK_MSG( IsOk(), 0, wxT("invalid font") );
 
@@ -544,7 +511,7 @@ wxFontStyle wxFont::GetStyle() const
     return M_FONTDATA->m_style;
 }
 
-wxFontWeight wxFont::GetWeight() const
+int wxFont::GetNumericWeight() const
 {
     wxCHECK_MSG( IsOk(), wxFONTWEIGHT_MAX, wxT("invalid font") );
 
@@ -601,11 +568,11 @@ bool wxFont::IsFixedWidth() const
 // change font attributes
 // ----------------------------------------------------------------------------
 
-void wxFont::SetPointSize(int pointSize)
+void wxFont::SetFractionalPointSize(double pointSize)
 {
     Unshare();
 
-    M_FONTDATA->SetPointSize(pointSize);
+    M_FONTDATA->SetFractionalPointSize(pointSize);
 }
 
 void wxFont::SetFamily(wxFontFamily family)
@@ -622,11 +589,11 @@ void wxFont::SetStyle(wxFontStyle style)
     M_FONTDATA->SetStyle(style);
 }
 
-void wxFont::SetWeight(wxFontWeight weight)
+void wxFont::SetNumericWeight(int weight)
 {
     Unshare();
 
-    M_FONTDATA->SetWeight(weight);
+    M_FONTDATA->SetNumericWeight(weight);
 }
 
 bool wxFont::SetFaceName(const wxString& faceName)
@@ -699,7 +666,7 @@ GdkFont *wxFont::GetInternalFont( float scale ) const
     wxCHECK_MSG( IsOk(), font, wxT("invalid font") );
 
     long int_scale = long(scale * 100.0 + 0.5); // key for fontlist
-    int point_scale = (int)((M_FONTDATA->m_pointSize * 10 * int_scale) / 100);
+    float point_scale = (M_FONTDATA->m_pointSize * 10 * scale) / 100.0;
 
     wxScaledFontList& list = M_FONTDATA->m_scaled_xfonts;
     wxScaledFontList::iterator i = list.find(int_scale);
