@@ -19,15 +19,6 @@
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
-
-#if wxUSE_DIRDLG
-
-#if wxUSE_OLE && !defined(__GNUWIN32_OLD__) && (!defined(__WXWINCE__) || \
-    (defined(__HANDHELDPC__) && (_WIN32_WCE >= 500)))
-
 #include "wx/dirdlg.h"
 #include "wx/modalhook.h"
 
@@ -38,142 +29,67 @@
     #include "wx/app.h"     // for GetComCtl32Version()
 #endif
 
-#include "wx/msw/private.h"
-#include "wx/msw/wrapshl.h"
-#include "wx/msw/private/comptr.h"
-#include "wx/dynlib.h"
+#include "wx/msw/private/filedialog.h"
+
+#if wxUSE_IFILEOPENDIALOG
 
 #include <initguid.h>
 
-// We can only use IFileDialog under desktop Windows and we need
-// wxDynamicLibrary for it.
-#if wxUSE_DYNLIB_CLASS && !defined(__WXWINCE__)
-    #define wxUSE_IFILEDIALOG 1
-#else
-    #define wxUSE_IFILEDIALOG 0
-#endif
+#include "wx/msw/private/cotaskmemptr.h"
+#include "wx/dynlib.h"
 
-#if wxUSE_IFILEDIALOG
 // IFileDialog related declarations missing from some compilers headers.
 
-// IShellItem
-#ifndef __IShellItem_INTERFACE_DEFINED__
-
-#ifndef SIGDN_FILESYSPATH
-    #define SIGDN_FILESYSPATH 0x80058000
-#endif
-
-struct IShellItem : public IUnknown
-{
-    virtual HRESULT wxSTDCALL BindToHandler(IBindCtx*, REFGUID, REFIID, void**) = 0;
-    virtual HRESULT wxSTDCALL GetParent(IShellItem**) = 0;
-    virtual HRESULT wxSTDCALL GetDisplayName(DWORD, LPWSTR*) = 0;
-    virtual HRESULT wxSTDCALL GetAttributes(ULONG, ULONG*) = 0;
-    virtual HRESULT wxSTDCALL Compare(IShellItem*, DWORD, int*) = 0;
-};
-
-#endif // #ifndef __IShellItem_INTERFACE_DEFINED__
-
-// Define this GUID in any case, even when __IShellItem_INTERFACE_DEFINED__ is
-// defined in the headers we might still not have it in the actual uuid.lib,
-// this happens with at least VC7 used with its original (i.e. not updated) SDK
-// and there is no harm in defining the GUID unconditionally.
+#if defined(__VISUALC__)
+// Always define this GUID, we might still not have it in the actual uuid.lib,
+// even when IShellItem interface is defined in the headers.
+// This happens with at least VC7 used with its original (i.e. not updated) SDK.
+// clang complains about multiple definitions, so only define it unconditionally
+// when using a Visual C compiler.
 DEFINE_GUID(IID_IShellItem,
     0x43826D1E, 0xE718, 0x42EE, 0xBC, 0x55, 0xA1, 0xE2, 0x61, 0xC3, 0x7B, 0xFE);
-
-struct IShellItemFilter;
-struct IFileDialogEvents;
-
-// IModalWindow
-#ifndef __IModalWindow_INTERFACE_DEFINED__
-
-struct IModalWindow : public IUnknown
-{
-    virtual HRESULT wxSTDCALL Show(HWND) = 0;
-};
-
-#endif // #ifndef __IModalWindow_INTERFACE_DEFINED__
-
-// IFileDialog
-#ifndef __IFileDialog_INTERFACE_DEFINED__
-
-#ifndef FOS_PICKFOLDERS
-    #define FOS_PICKFOLDERS 0x20
 #endif
 
-#ifndef FOS_FORCEFILESYSTEM
-    #define FOS_FORCEFILESYSTEM 0x40
-#endif
+#endif // wxUSE_IFILEOPENDIALOG
 
-struct _COMDLG_FILTERSPEC;
+// ----------------------------------------------------------------------------
+// private functions prototypes
+// ----------------------------------------------------------------------------
 
-struct IFileDialog : public IModalWindow
+#if wxUSE_IFILEOPENDIALOG
+
+namespace
 {
-    virtual HRESULT wxSTDCALL SetFileTypes(UINT, const _COMDLG_FILTERSPEC*) = 0;
-    virtual HRESULT wxSTDCALL SetFileTypeIndex(UINT) = 0;
-    virtual HRESULT wxSTDCALL GetFileTypeIndex(UINT*) = 0;
-    virtual HRESULT wxSTDCALL Advise(IFileDialogEvents*, DWORD*) = 0;
-    virtual HRESULT wxSTDCALL Unadvise(DWORD) = 0;
-    virtual HRESULT wxSTDCALL SetOptions(DWORD) = 0;
-    virtual HRESULT wxSTDCALL GetOptions(DWORD*) = 0;
-    virtual HRESULT wxSTDCALL SetDefaultFolder(IShellItem*) = 0;
-    virtual HRESULT wxSTDCALL SetFolder(IShellItem*) = 0;
-    virtual HRESULT wxSTDCALL GetFolder(IShellItem**) = 0;
-    virtual HRESULT wxSTDCALL GetCurrentSelection(IShellItem**) = 0;
-    virtual HRESULT wxSTDCALL SetFileName(LPCWSTR) = 0;
-    virtual HRESULT wxSTDCALL GetFileName(LPWSTR*) = 0;
-    virtual HRESULT wxSTDCALL SetTitle(LPCWSTR) = 0;
-    virtual HRESULT wxSTDCALL SetOkButtonLabel(LPCWSTR) = 0;
-    virtual HRESULT wxSTDCALL SetFileNameLabel(LPCWSTR) = 0;
-    virtual HRESULT wxSTDCALL GetResult(IShellItem**) = 0;
-    virtual HRESULT wxSTDCALL AddPlace(IShellItem*, DWORD) = 0;
-    virtual HRESULT wxSTDCALL SetDefaultExtension(LPCWSTR) = 0;
-    virtual HRESULT wxSTDCALL Close(HRESULT) = 0;
-    virtual HRESULT wxSTDCALL SetClientGuid(REFGUID) = 0;
-    virtual HRESULT wxSTDCALL ClearClientData() = 0;
-    virtual HRESULT wxSTDCALL SetFilter(IShellItemFilter*) = 0;
-};
 
-DEFINE_GUID(CLSID_FileOpenDialog,
-    0xDC1C5A9C, 0xE88A, 0x4dde, 0xA5, 0xA1, 0x60, 0xF8, 0x2A, 0x20, 0xAE, 0xF7);
+// helper functions for wxDirDialog::ShowIFileOpenDialog()
+bool GetPathsFromIFileOpenDialog(IFileOpenDialog* fileDialog, wxArrayString& paths);
+bool GetPathFromIFileDialog(IFileDialog* fileDialog, wxString& path);
 
-DEFINE_GUID(IID_IFileDialog,
-    0x42F85136, 0xDB7E, 0x439C, 0x85, 0xF1, 0xE4, 0x07, 0x5D, 0x13, 0x5F, 0xC8);
+} // anonymous namespace
 
-#endif // #ifndef __IFileDialog_INTERFACE_DEFINED__
+#endif // #if wxUSE_IFILEOPENDIALOG
 
-#endif // wxUSE_IFILEDIALOG
+// Note that parts of this file related to IFileDialog are still compiled even
+// when wxUSE_DIRDLG == 0 because they're used by wxUSE_FILEDLG too.
+#if wxUSE_DIRDLG
+
+// callback used in wxDirDialog::ShowSHBrowseForFolder()
+static int CALLBACK BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lp,
+                                       LPARAM pData);
 
 // ----------------------------------------------------------------------------
 // constants
 // ----------------------------------------------------------------------------
 
-#ifndef BIF_NEWDIALOGSTYLE
-    #define BIF_NEWDIALOGSTYLE 0x0040
-#endif
-
 #ifndef BIF_NONEWFOLDERBUTTON
     #define BIF_NONEWFOLDERBUTTON  0x0200
 #endif
 
-#ifndef BIF_EDITBOX
-    #define BIF_EDITBOX 16
-#endif
-
 // ----------------------------------------------------------------------------
 // wxWidgets macros
 // ----------------------------------------------------------------------------
 
-IMPLEMENT_CLASS(wxDirDialog, wxDialog)
-
-// ----------------------------------------------------------------------------
-// private functions prototypes
-// ----------------------------------------------------------------------------
-
-// the callback proc for the dir dlg
-static int CALLBACK BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lp,
-                                       LPARAM pData);
-
+wxIMPLEMENT_CLASS(wxDirDialog, wxDialog);
 
 // ============================================================================
 // implementation
@@ -194,6 +110,9 @@ wxDirDialog::wxDirDialog(wxWindow *parent,
     m_message = message;
     m_parent = parent;
 
+    wxASSERT_MSG( !( (style & wxDD_MULTIPLE) && (style & wxDD_CHANGE_DIR) ),
+                  "wxDD_CHANGE_DIR can't be used together with wxDD_MULTIPLE" );
+
     SetWindowStyle(style);
     SetPath(defaultPath);
 }
@@ -225,9 +144,13 @@ int wxDirDialog::ShowModal()
     wxWindow* const parent = GetParentForModalDialog();
     WXHWND hWndParent = parent ? GetHwndOf(parent) : NULL;
 
+    wxWindowDisabler disableOthers(this, parent);
+
+    m_paths.clear();
+
     // Use IFileDialog under new enough Windows, it's more user-friendly.
     int rc;
-#if wxUSE_IFILEDIALOG
+#if wxUSE_IFILEOPENDIALOG
     // While the new dialog is available under Vista, it may return a wrong
     // path there (see http://support.microsoft.com/kb/969885/en-us), so we
     // don't use it there by default. We could improve the version test to
@@ -235,7 +158,7 @@ int wxDirDialog::ShowModal()
     // as this means that the hotfix correcting this bug is installed.
     if ( wxGetWinVersion() > wxWinVersion_Vista )
     {
-        rc = ShowIFileDialog(hWndParent);
+        rc = ShowIFileOpenDialog(hWndParent);
     }
     else
     {
@@ -243,7 +166,7 @@ int wxDirDialog::ShowModal()
     }
 
     if ( rc == wxID_NONE )
-#endif // wxUSE_IFILEDIALOG
+#endif // wxUSE_IFILEOPENDIALOG
     {
         rc = ShowSHBrowseForFolder(hWndParent);
     }
@@ -261,32 +184,21 @@ int wxDirDialog::ShowSHBrowseForFolder(WXHWND owner)
     bi.hwndOwner      = owner;
     bi.pidlRoot       = NULL;
     bi.pszDisplayName = NULL;
-    // Please don't change this without checking it compiles
-    // with eVC++ first.
-#if defined(__POCKETPC__) || defined(__SMARTPHONE__)
-    bi.lpszTitle      = m_message.mb_str();
-#else
     bi.lpszTitle      = m_message.c_str();
-#endif
     bi.ulFlags        = BIF_RETURNONLYFSDIRS | BIF_STATUSTEXT;
     bi.lpfn           = BrowseCallbackProc;
     bi.lParam         = wxMSW_CONV_LPARAM(m_path); // param for the callback
 
     static const int verComCtl32 = wxApp::GetComCtl32Version();
 
-    // we always add the edit box (it doesn't hurt anybody, does it?) if it is
-    // supported by the system
-    if ( verComCtl32 >= 471 )
-    {
-        bi.ulFlags |= BIF_EDITBOX;
-    }
+    // we always add the edit box (it doesn't hurt anybody, does it?)
+    bi.ulFlags |= BIF_EDITBOX;
 
     // to have the "New Folder" button we must use the "new" dialog style which
     // is also the only way to have a resizable dialog
     //
-    // "new" style is only available in the version 5.0+ of comctl32.dll
     const bool needNewDir = !HasFlag(wxDD_DIR_MUST_EXIST);
-    if ( (needNewDir || HasFlag(wxRESIZE_BORDER)) && (verComCtl32 >= 500) )
+    if ( needNewDir || HasFlag(wxRESIZE_BORDER) )
     {
         if (needNewDir)
         {
@@ -309,7 +221,7 @@ int wxDirDialog::ShowSHBrowseForFolder(WXHWND owner)
     // do show the dialog
     wxItemIdList pidl(SHBrowseForFolder(&bi));
 
-    wxItemIdList::Free((LPITEMIDLIST)bi.pidlRoot);
+    wxItemIdList::Free(const_cast<LPITEMIDLIST>(bi.pidlRoot));
 
     if ( !pidl )
     {
@@ -326,152 +238,288 @@ int wxDirDialog::ShowSHBrowseForFolder(WXHWND owner)
 //
 // Returns wxID_OK on success, wxID_CANCEL if cancelled by user or wxID_NONE if
 // an error occurred and we should fall back onto the old dialog.
-#if wxUSE_IFILEDIALOG
+#if wxUSE_IFILEOPENDIALOG
 
-int wxDirDialog::ShowIFileDialog(WXHWND owner)
+int wxDirDialog::ShowIFileOpenDialog(WXHWND owner)
 {
-    HRESULT hr;
-    wxCOMPtr<IFileDialog> fileDialog;
+    wxMSWImpl::wxIFileDialog fileDialog(CLSID_FileOpenDialog);
+    if ( !fileDialog.IsOk() )
+        return wxID_NONE;
 
-    hr = ::CoCreateInstance(CLSID_FileOpenDialog, NULL, CLSCTX_INPROC_SERVER,
-                            wxIID_PPV_ARGS(IFileDialog, &fileDialog));
+    fileDialog.SetTitle(m_message);
+    if ( !m_path.empty() )
+        fileDialog.SetInitialPath(m_path);
+
+    // We currently always use FOS_NOCHANGEDIR even if wxDD_CHANGE_DIR was
+    // specified because we change the directory ourselves in this case.
+    int options = FOS_PICKFOLDERS | FOS_NOCHANGEDIR;
+    if ( HasFlag(wxDD_MULTIPLE) )
+        options |= FOS_ALLOWMULTISELECT;
+    if ( HasFlag(wxDD_SHOW_HIDDEN) )
+        options |= FOS_FORCESHOWHIDDEN;
+
+    return fileDialog.Show(owner, options, &m_paths, &m_path);
+}
+
+#endif // wxUSE_IFILEOPENDIALOG
+
+#endif // wxUSE_DIRDLG
+
+#if wxUSE_IFILEOPENDIALOG
+
+// ----------------------------------------------------------------------------
+// Helper functions used by wxDirDialog and wxFileDialog.
+// ----------------------------------------------------------------------------
+
+namespace wxMSWImpl
+{
+
+wxIFileDialog::wxIFileDialog(const CLSID& clsid)
+{
+    HRESULT hr = ::CoCreateInstance
+                 (
+                    clsid,
+                    NULL, // no outer IUnknown
+                    CLSCTX_INPROC_SERVER,
+                    wxIID_PPV_ARGS(IFileDialog, &m_fileDialog)
+                 );
     if ( FAILED(hr) )
     {
         wxLogApiError(wxS("CoCreateInstance(CLSID_FileOpenDialog)"), hr);
-        return wxID_NONE;
     }
+}
 
-    // allow user to select only a file system folder
-    hr = fileDialog->SetOptions(FOS_PICKFOLDERS | FOS_FORCEFILESYSTEM);
+int wxIFileDialog::Show(HWND owner, int options,
+                        wxArrayString* pathsOut, wxString* pathOut)
+{
+    wxCHECK_MSG( m_fileDialog, wxID_NONE, wxS("shouldn't be called") );
+
+    HRESULT hr;
+
+    // allow to select only a file system object
+    hr = m_fileDialog->SetOptions(options | FOS_FORCEFILESYSTEM);
     if ( FAILED(hr) )
     {
         wxLogApiError(wxS("IFileDialog::SetOptions"), hr);
-        return wxID_NONE;
+        return false;
     }
 
-    hr = fileDialog->SetTitle(m_message.wc_str());
+    hr = m_fileDialog->Show(owner);
+    if ( FAILED(hr) )
+    {
+        if ( hr == HRESULT_FROM_WIN32(ERROR_CANCELLED) )
+        {
+            return wxID_CANCEL; // the user cancelled the dialog
+        }
+        else
+        {
+            wxLogApiError(wxS("IFileDialog::Show"), hr);
+        }
+    }
+    else if ( options & FOS_ALLOWMULTISELECT )
+    {
+        wxCOMPtr<IFileOpenDialog> fileOpenDialog;
+        hr = m_fileDialog->QueryInterface(wxIID_PPV_ARGS(IFileOpenDialog, &fileOpenDialog));
+        if ( SUCCEEDED(hr) )
+        {
+            if ( GetPathsFromIFileOpenDialog(fileOpenDialog, *pathsOut) )
+                return wxID_OK;
+        }
+        else
+        {
+            wxLogApiError(wxS("IFileDialog::QI(IFileOpenDialog)"), hr);
+        }
+    }
+    else // Single selection only, path output parameter must be non-null.
+    {
+        if ( GetPathFromIFileDialog(m_fileDialog, *pathOut) )
+            return wxID_OK;
+    }
+
+    // Failed to show the dialog or obtain the selected folders(s)
+    wxLogSysError(_("Couldn't obtain folder name"), hr);
+    return wxID_CANCEL;
+}
+
+void wxIFileDialog::SetTitle(const wxString& message)
+{
+    HRESULT hr = m_fileDialog->SetTitle(message.wc_str());
     if ( FAILED(hr) )
     {
         // This error is not serious, let's just log it and continue even
         // without the title set.
         wxLogApiError(wxS("IFileDialog::SetTitle"), hr);
     }
+}
 
-    // set the initial path
-    if ( !m_path.empty() )
+void wxIFileDialog::SetInitialPath(const wxString& defaultPath)
+{
+    HRESULT hr;
+
+    // We need to link SHCreateItemFromParsingName() dynamically as it's
+    // not available on pre-Vista systems.
+    typedef HRESULT
+    (WINAPI *SHCreateItemFromParsingName_t)(PCWSTR,
+                                            IBindCtx*,
+                                            REFIID,
+                                            void**);
+
+    static SHCreateItemFromParsingName_t
+        s_pfnSHCreateItemFromParsingName = (SHCreateItemFromParsingName_t)-1;
+    if ( s_pfnSHCreateItemFromParsingName == (SHCreateItemFromParsingName_t)-1 )
     {
-        // We need to link SHCreateItemFromParsingName() dynamically as it's
-        // not available on pre-Vista systems.
-        typedef HRESULT
-        (WINAPI *SHCreateItemFromParsingName_t)(PCWSTR,
-                                                IBindCtx*,
-                                                REFIID,
-                                                void**);
-
-        SHCreateItemFromParsingName_t pfnSHCreateItemFromParsingName = NULL;
         wxDynamicLibrary dllShell32;
         if ( dllShell32.Load(wxS("shell32.dll"), wxDL_VERBATIM | wxDL_QUIET) )
         {
-            wxDL_INIT_FUNC(pfn, SHCreateItemFromParsingName, dllShell32);
+            wxDL_INIT_FUNC(s_pfn, SHCreateItemFromParsingName, dllShell32);
         }
 
-        if ( !pfnSHCreateItemFromParsingName )
+        if ( !s_pfnSHCreateItemFromParsingName )
         {
             wxLogLastError(wxS("SHCreateItemFromParsingName() not found"));
-            return wxID_NONE;
-        }
-
-        wxCOMPtr<IShellItem> folder;
-        hr = pfnSHCreateItemFromParsingName(m_path.wc_str(),
-                                              NULL,
-                                              wxIID_PPV_ARGS(IShellItem,
-                                                             &folder));
-
-        // Failing to parse the folder name is not really an error, we'll just
-        // ignore the initial directory in this case, but we should still show
-        // the dialog.
-        if ( FAILED(hr) )
-        {
-            if ( hr != HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND) )
-            {
-                wxLogApiError(wxS("SHCreateItemFromParsingName"), hr);
-                return wxID_NONE;
-            }
-        }
-        else // The folder was parsed correctly.
-        {
-            hr = fileDialog->SetFolder(folder);
-            if ( FAILED(hr) )
-            {
-                wxLogApiError(wxS("IFileDialog::SetFolder"), hr);
-                return wxID_NONE;
-            }
         }
     }
 
+    if ( !s_pfnSHCreateItemFromParsingName )
+    {
+        // There is nothing we can do and the error was already reported.
+        return;
+    }
 
-    wxString path;
+    wxCOMPtr<IShellItem> folder;
+    hr = s_pfnSHCreateItemFromParsingName
+         (
+            defaultPath.wc_str(),
+            NULL,
+            wxIID_PPV_ARGS(IShellItem, &folder)
+         );
 
-    hr = fileDialog->Show(owner);
+    // Failing to parse the folder name or set it is not really an error,
+    // we'll just ignore the initial directory in this case, but we should
+    // still show the dialog.
     if ( SUCCEEDED(hr) )
     {
-        wxCOMPtr<IShellItem> folder;
-
-        hr = fileDialog->GetResult(&folder);
-        if ( SUCCEEDED(hr) )
-        {
-            LPOLESTR pathOLE = NULL;
-
-            hr = folder->GetDisplayName(SIGDN_FILESYSPATH, &pathOLE);
-            if ( SUCCEEDED(hr) )
-            {
-                path = pathOLE;
-                CoTaskMemFree(pathOLE);
-            }
-            else
-            {
-                wxLogApiError(wxS("IShellItem::GetDisplayName"), hr);
-            }
-        }
-        else
-        {
-            wxLogApiError(wxS("IFileDialog::GetResult"), hr);
-        }
+        hr = m_fileDialog->SetFolder(folder);
+        if ( FAILED(hr) )
+            wxLogApiError(wxS("IFileDialog::SetFolder"), hr);
     }
-    else if ( hr == HRESULT_FROM_WIN32(ERROR_CANCELLED) )
-    {
-        return wxID_CANCEL; // the user cancelled the dialog
-    }
-    else
-    {
-        wxLogApiError(wxS("IFileDialog::Show"), hr);
-    }
-
-    if ( path.empty() )
-    {
-        // the user didn't cancel the dialog and yet the path is empty
-        // it means there was an error, already logged by wxLogApiError()
-        // now report the error to the user and return
-        wxLogSysError(_("Couldn't obtain folder name"), hr);
-        return wxID_CANCEL;
-    }
-
-    m_path = path;
-    return wxID_OK;
 }
 
-#endif // wxUSE_IFILEDIALOG
+} // namespace wxMSWImpl
 
 // ----------------------------------------------------------------------------
 // private functions
 // ----------------------------------------------------------------------------
 
+namespace
+{
+
+// helper function for wxDirDialog::ShowIFileOpenDialog()
+bool GetPathsFromIFileOpenDialog(IFileOpenDialog* fileDialog, wxArrayString& paths)
+{
+    HRESULT hr = S_OK;
+    wxString path;
+    wxArrayString tempPaths;
+
+    wxCOMPtr<IShellItemArray> itemArray;
+
+    hr = fileDialog->GetResults(&itemArray);
+    if ( FAILED(hr) )
+    {
+        wxLogApiError(wxS("IShellItemArray::GetResults"), hr);
+        return false;
+    }
+
+    DWORD count = 0;
+
+    hr = itemArray->GetCount(&count);
+    if ( FAILED(hr) )
+    {
+        wxLogApiError(wxS("IShellItemArray::GetCount"), hr);
+        return false;
+    }
+
+    for ( DWORD i = 0; i < count; ++i )
+    {
+        wxCOMPtr<IShellItem> item;
+
+        hr = itemArray->GetItemAt(i, &item);
+        if ( FAILED(hr) )
+        {
+            // do not attempt to retrieve any other items
+            // and just fail
+            wxLogApiError(wxS("IShellItemArray::GetItem"), hr);
+            tempPaths.clear();
+            break;
+        }
+
+        hr = wxMSWImpl::GetFSPathFromShellItem(item, path);
+        if ( FAILED(hr) )
+        {
+            // again, just fail
+            tempPaths.clear();
+            break;
+        }
+
+        tempPaths.push_back(path);
+    }
+
+    if ( tempPaths.empty() )
+        return false; // there was en error
+
+    paths = tempPaths;
+    return true;
+}
+
+bool GetPathFromIFileDialog(IFileDialog* fileDialog, wxString& path)
+{
+    wxCOMPtr<IShellItem> item;
+
+    HRESULT hr = fileDialog->GetResult(&item);
+    if ( FAILED(hr) )
+    {
+        wxLogApiError(wxS("IFileDialog::GetResult"), hr);
+        return false;
+    }
+
+    hr = wxMSWImpl::GetFSPathFromShellItem(item, path);
+    if ( FAILED(hr) )
+    {
+        return false;
+    }
+
+    return true;
+}
+
+} // anonymous namespace
+
+HRESULT
+wxMSWImpl::GetFSPathFromShellItem(const wxCOMPtr<IShellItem>& item, wxString& path)
+{
+    wxCoTaskMemPtr<WCHAR> pOLEPath;
+    const HRESULT hr = item->GetDisplayName(SIGDN_FILESYSPATH, &pOLEPath);
+
+    if ( FAILED(hr) )
+    {
+        wxLogApiError(wxS("IShellItem::GetDisplayName"), hr);
+        return hr;
+    }
+
+    path = pOLEPath;
+
+    return S_OK;
+}
+
+#endif // wxUSE_IFILEOPENDIALOG
+
+#if wxUSE_DIRDLG
+
+// callback used in wxDirDialog::ShowSHBrowseForFolder()
 static int CALLBACK
 BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lp, LPARAM pData)
 {
     switch(uMsg)
     {
-#ifdef BFFM_SETSELECTION
         case BFFM_INITIALIZED:
             // sent immediately after initialisation and so we may set the
             // initial selection here
@@ -479,8 +527,6 @@ BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lp, LPARAM pData)
             // wParam = TRUE => lParam is a string and not a PIDL
             ::SendMessage(hwnd, BFFM_SETSELECTION, TRUE, pData);
             break;
-#endif // BFFM_SETSELECTION
-
 
         case BFFM_SELCHANGED:
             // note that this doesn't work with the new style UI (MSDN doesn't
@@ -518,6 +564,4 @@ BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lp, LPARAM pData)
     return 0;
 }
 
-#endif // compiler/platform on which the code here compiles
-
 #endif // wxUSE_DIRDLG
