@@ -3,7 +3,7 @@
 // Purpose:     wxTextEntry implementation for wxMSW
 // Author:      Vadim Zeitlin
 // Created:     2007-09-26
-// Copyright:   (c) 2007 Vadim Zeitlin <vadim@wxwindows.org>
+// Copyright:   (c) 2007 Vadim Zeitlin <vadim@wxwidgets.org>
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -18,13 +18,12 @@
 // for compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 #ifndef WX_PRECOMP
     #include "wx/arrstr.h"
+    #include "wx/event.h"
     #include "wx/string.h"
+    #include "wx/textctrl.h"    // Only for wxTE_PROCESS_XXX constants
 #endif // WX_PRECOMP
 
 #if wxUSE_TEXTCTRL || wxUSE_COMBOBOX
@@ -34,19 +33,23 @@
 #include "wx/dynlib.h"
 
 #include "wx/msw/private.h"
+#include "wx/msw/private/winstyle.h"
+#include "wx/msw/private/cotaskmemptr.h"
 
 #if wxUSE_UXTHEME
     #include "wx/msw/uxtheme.h"
 #endif
 
+#include "wx/msw/wrapwin.h"
+#include <shlwapi.h>
+
 #define GetEditHwnd() ((HWND)(GetEditHWND()))
 
 // ----------------------------------------------------------------------------
 // Classes used by auto-completion implementation.
 // ----------------------------------------------------------------------------
 
-// standard VC6 SDK (WINVER == 0x0400) does not know about IAutoComplete
-#if wxUSE_OLE && (WINVER >= 0x0500)
+#if wxUSE_OLE
     #define HAS_AUTOCOMPLETE
 #endif
 
@@ -55,7 +58,7 @@
 #include "wx/msw/ole/oleutils.h"
 #include <shldisp.h>
 
-#if defined(__MINGW32__) || defined (__WATCOMC__) || defined(__CYGWIN__)
+#if defined(__MINGW32__) || defined(__CYGWIN__)
     // needed for IID_IAutoComplete, IID_IAutoComplete2 and ACO_AUTOSUGGEST
     #include <shlguid.h>
 
@@ -64,10 +67,6 @@
     #endif
 #endif
 
-#ifndef ACO_UPDOWNKEYDROPSLIST
-    #define ACO_UPDOWNKEYDROPSLIST 0x20
-#endif
-
 #ifndef SHACF_FILESYS_ONLY
     #define SHACF_FILESYS_ONLY 0x00000010
 #endif
@@ -159,21 +158,26 @@ public:
         m_completer = completer;
     }
 
-    void UpdatePrefix(const wxString& prefix)
+    bool UpdatePrefix(const wxString& prefix)
     {
         CSLock lock(m_csRestart);
 
+        if ( prefix == m_prefix )
+            return false;
+
         // We simply store the prefix here and will really update during the
         // next call to our Next() method as we want to call Start() from the
         // worker thread to prevent the main UI thread from blocking while the
         // completions are generated.
         m_prefix = prefix;
         m_restart = TRUE;
+
+        return true;
     }
 
     virtual HRESULT STDMETHODCALLTYPE Next(ULONG celt,
                                            LPOLESTR *rgelt,
-                                           ULONG *pceltFetched)
+                                           ULONG *pceltFetched) wxOVERRIDE
     {
         if ( !rgelt || (!pceltFetched && celt > 1) )
             return E_POINTER;
@@ -201,13 +205,13 @@ public:
 
             const wxWX2WCbuf wcbuf = s.wc_str();
             const size_t size = (wcslen(wcbuf) + 1)*sizeof(wchar_t);
-            void *olestr = CoTaskMemAlloc(size);
+            wxCoTaskMemPtr<wchar_t> olestr(size);
+
             if ( !olestr )
                 return E_OUTOFMEMORY;
 
             memcpy(olestr, wcbuf, size);
-
-            *rgelt++ = static_cast<LPOLESTR>(olestr);
+            *rgelt++ = olestr.release();
 
             ++(*pceltFetched);
         }
@@ -215,7 +219,7 @@ public:
         return S_OK;
     }
 
-    virtual HRESULT STDMETHODCALLTYPE Skip(ULONG celt)
+    virtual HRESULT STDMETHODCALLTYPE Skip(ULONG celt) wxOVERRIDE
     {
         if ( !celt )
             return E_INVALIDARG;
@@ -237,7 +241,7 @@ public:
         return S_OK;
     }
 
-    virtual HRESULT STDMETHODCALLTYPE Reset()
+    virtual HRESULT STDMETHODCALLTYPE Reset() wxOVERRIDE
     {
         CSLock lock(m_csRestart);
 
@@ -246,7 +250,7 @@ public:
         return S_OK;
     }
 
-    virtual HRESULT STDMETHODCALLTYPE Clone(IEnumString **ppEnum)
+    virtual HRESULT STDMETHODCALLTYPE Clone(IEnumString **ppEnum) wxOVERRIDE
     {
         if ( !ppEnum )
             return E_POINTER;
@@ -357,7 +361,7 @@ IMPLEMENT_IUNKNOWN_METHODS(wxIEnumString)
 
 // This class gathers the all auto-complete-related stuff we use. It is
 // allocated on demand by wxTextEntry when AutoComplete() is called.
-class wxTextAutoCompleteData wxBIND_OR_CONNECT_HACK_ONLY_BASE_CLASS
+class wxTextAutoCompleteData
 {
 public:
     // The constructor associates us with the given text entry.
@@ -438,9 +442,7 @@ public:
             pAutoComplete2->Release();
         }
 
-        wxBIND_OR_CONNECT_HACK(m_win, wxEVT_CHAR_HOOK, wxKeyEventHandler,
-                               wxTextAutoCompleteData::OnCharHook,
-                               this);
+        m_win->Bind(wxEVT_CHAR_HOOK, &wxTextAutoCompleteData::OnCharHook, this);
     }
 
     ~wxTextAutoCompleteData()
@@ -506,10 +508,8 @@ public:
                 // neither as, due to our use of ACO_AUTOAPPEND, we get
                 // EN_CHANGE notifications from the control every time
                 // IAutoComplete auto-appends something to it.
-                wxBIND_OR_CONNECT_HACK(m_win, wxEVT_AFTER_CHAR,
-                                        wxKeyEventHandler,
-                                        wxTextAutoCompleteData::OnAfterChar,
-                                        this);
+                m_win->Bind(wxEVT_AFTER_CHAR,
+                            &wxTextAutoCompleteData::OnAfterChar, this);
             }
 
             UpdateStringsFromCustomCompleter();
@@ -560,39 +560,71 @@ private:
 
         const wxString prefix = m_entry->GetRange(0, from);
 
-        m_enumStrings->UpdatePrefix(prefix);
-
-        DoRefresh();
+        if ( m_enumStrings->UpdatePrefix(prefix) )
+            DoRefresh();
     }
 
     void OnAfterChar(wxKeyEvent& event)
     {
-        // Notice that we must not refresh the completions when the user
-        // presses Backspace as this would result in adding back the just
-        // erased character(s) because of ACO_AUTOAPPEND option we use.
-        if ( m_customCompleter && event.GetKeyCode() != WXK_BACK )
+        if ( m_customCompleter )
+        {
             UpdateStringsFromCustomCompleter();
+        }
 
         event.Skip();
     }
 
     void OnCharHook(wxKeyEvent& event)
     {
-        // If the autocomplete drop-down list is currently displayed when the
-        // user presses Escape, we need to dismiss it manually from here as
-        // Escape could be eaten by something else (e.g. EVT_CHAR_HOOK in the
-        // dialog that this control is found in) otherwise.
-        if ( event.GetKeyCode() == WXK_ESCAPE )
+        // We need to override the default handling of some keys here.
+        bool specialKey = false;
+        switch ( event.GetKeyCode() )
         {
+            case WXK_RETURN:
+            case WXK_NUMPAD_ENTER:
+                if ( m_win->HasFlag(wxTE_PROCESS_ENTER) )
+                    specialKey = true;
+                break;
+
+            case WXK_TAB:
+                if ( m_win->HasFlag(wxTE_PROCESS_TAB) )
+                    specialKey = true;
+                break;
+
+            case WXK_ESCAPE:
+                specialKey = true;
+                break;
+        }
+
+        if ( specialKey )
+        {
+            // Check if the drop down is currently open.
             DWORD dwFlags = 0;
             if ( SUCCEEDED(m_autoCompleteDropDown->GetDropDownStatus(&dwFlags,
                                                                      NULL))
                     && dwFlags == ACDD_VISIBLE )
             {
-                ::SendMessage(GetHwndOf(m_win), WM_KEYDOWN, WXK_ESCAPE, 0);
+                if ( event.GetKeyCode() == WXK_ESCAPE )
+                {
+                    // We need to dismiss the drop-down manually as Escape
+                    // could be eaten by something else (e.g. EVT_CHAR_HOOK in
+                    // the dialog that this control is found in) otherwise.
+                    ::SendMessage(GetHwndOf(m_win), WM_KEYDOWN, WXK_ESCAPE, 0);
 
-                // Do not skip the event in this case, we've already handled it.
-                return;
+                    // Do not skip the event in this case, we've already handled it.
+                    return;
+                }
+            }
+            else // Drop down is not open.
+            {
+                // In this case we need to handle Return and Tab as both of
+                // them are simply eaten by the auto completer and never reach
+                // us at all otherwise.
+                if ( event.GetKeyCode() != WXK_ESCAPE )
+                {
+                    m_entry->MSWProcessSpecialKey(event);
+                    return;
+                }
             }
         }
 
@@ -627,6 +659,10 @@ private:
     wxDECLARE_NO_COPY_CLASS(wxTextAutoCompleteData);
 };
 
+// Special pointer value which indicates that we're using SHAutoComplete().
+static wxTextAutoCompleteData* const wxDUMMY_SHAUTOCOMPLETE_DATA =
+    reinterpret_cast<wxTextAutoCompleteData*>(-1);
+
 #endif // HAS_AUTOCOMPLETE
 
 // ============================================================================
@@ -647,7 +683,8 @@ wxTextEntry::wxTextEntry()
 wxTextEntry::~wxTextEntry()
 {
 #ifdef HAS_AUTOCOMPLETE
-    delete m_autoCompleteData;
+    if ( MSWHasAutoCompleteData() )
+        delete m_autoCompleteData;
 #endif // HAS_AUTOCOMPLETE
 }
 
@@ -780,24 +817,6 @@ void wxTextEntry::GetSelection(long *from, long *to) const
 
 bool wxTextEntry::DoAutoCompleteFileNames(int flags)
 {
-    typedef HRESULT (WINAPI *SHAutoComplete_t)(HWND, DWORD);
-    static SHAutoComplete_t s_pfnSHAutoComplete = (SHAutoComplete_t)-1;
-    static wxDynamicLibrary s_dllShlwapi;
-    if ( s_pfnSHAutoComplete == (SHAutoComplete_t)-1 )
-    {
-        if ( !s_dllShlwapi.Load(wxT("shlwapi.dll"), wxDL_VERBATIM | wxDL_QUIET) )
-        {
-            s_pfnSHAutoComplete = NULL;
-        }
-        else
-        {
-            wxDL_INIT_FUNC(s_pfn, SHAutoComplete, s_dllShlwapi);
-        }
-    }
-
-    if ( !s_pfnSHAutoComplete )
-        return false;
-
     DWORD dwFlags = 0;
     if ( flags & wxFILE )
         dwFlags |= SHACF_FILESYS_ONLY;
@@ -809,7 +828,7 @@ bool wxTextEntry::DoAutoCompleteFileNames(int flags)
         return false;
     }
 
-    HRESULT hr = (*s_pfnSHAutoComplete)(GetEditHwnd(), dwFlags);
+    HRESULT hr = ::SHAutoComplete(GetEditHwnd(), dwFlags);
     if ( FAILED(hr) )
     {
         wxLogApiError(wxT("SHAutoComplete()"), hr);
@@ -819,35 +838,59 @@ bool wxTextEntry::DoAutoCompleteFileNames(int flags)
 
     // Disable the other kinds of completion now that we use the built-in file
     // names completion.
-    if ( m_autoCompleteData )
-        m_autoCompleteData->DisableCompletion();
+    if ( MSWHasAutoCompleteData() )
+        delete m_autoCompleteData;
+
+    // Set it to the special value indicating that we're using SHAutoComplete().
+    m_autoCompleteData = wxDUMMY_SHAUTOCOMPLETE_DATA;
 
     return true;
 }
 
 #endif // wxUSE_DYNLIB_CLASS
 
-wxTextAutoCompleteData *wxTextEntry::GetOrCreateCompleter()
+void wxTextEntry::MSWProcessSpecialKey(wxKeyEvent& WXUNUSED(event))
 {
-    if ( !m_autoCompleteData )
+    wxFAIL_MSG(wxS("Must be overridden if can be called"));
+}
+
+bool wxTextEntry::MSWUsesStandardAutoComplete() const
+{
+    return m_autoCompleteData == wxDUMMY_SHAUTOCOMPLETE_DATA;
+}
+
+bool wxTextEntry::MSWHasAutoCompleteData() const
+{
+    // We use special wxDUMMY_SHAUTOCOMPLETE_DATA for the pointer to indicate
+    // that we're using SHAutoComplete(), so we need to check for it too, and
+    // not just whether the pointer is non-NULL.
+    return m_autoCompleteData != NULL
+            && m_autoCompleteData != wxDUMMY_SHAUTOCOMPLETE_DATA;
+}
+
+bool wxTextEntry::MSWEnsureHasAutoCompleteData()
+{
+    if ( !MSWHasAutoCompleteData() )
     {
         wxTextAutoCompleteData * const ac = new wxTextAutoCompleteData(this);
-        if ( ac->IsOk() )
-            m_autoCompleteData = ac;
-        else
+        if ( !ac->IsOk() )
+        {
             delete ac;
+            return false;
+        }
+
+        m_autoCompleteData = ac;
     }
 
-    return m_autoCompleteData;
+    return true;
 }
 
 bool wxTextEntry::DoAutoCompleteStrings(const wxArrayString& choices)
 {
-    wxTextAutoCompleteData * const ac = GetOrCreateCompleter();
-    if ( !ac )
+    if ( !MSWEnsureHasAutoCompleteData() )
         return false;
 
-    ac->ChangeStrings(choices);
+    m_autoCompleteData->ChangeStrings(choices);
 
     return true;
 }
@@ -857,14 +900,13 @@ bool wxTextEntry::DoAutoCompleteCustom(wxTextCompleter *completer)
     // First deal with the case when we just want to disable auto-completion.
     if ( !completer )
     {
-        if ( m_autoCompleteData )
+        if ( MSWHasAutoCompleteData() )
             m_autoCompleteData->DisableCompletion();
         //else: Nothing to do, we hadn't used auto-completion even before.
     }
     else // Have a valid completer.
     {
-        wxTextAutoCompleteData * const ac = GetOrCreateCompleter();
-        if ( !ac )
+        if ( !MSWEnsureHasAutoCompleteData() )
         {
             // Delete the custom completer for consistency with the case when
             // we succeed to avoid memory leaks in user code.
@@ -873,7 +915,7 @@ bool wxTextEntry::DoAutoCompleteCustom(wxTextCompleter *completer)
         }
 
         // This gives ownership of the custom completer to m_autoCompleteData.
-        if ( !ac->ChangeCustomCompleter(completer) )
+        if ( !m_autoCompleteData->ChangeCustomCompleter(completer) )
             return false;
     }
 
@@ -918,7 +960,7 @@ void wxTextEntry::SetEditable(bool editable)
 }
 
 // ----------------------------------------------------------------------------
-// max length
+// input restrictions
 // ----------------------------------------------------------------------------
 
 void wxTextEntry::SetMaxLength(unsigned long len)
@@ -933,6 +975,13 @@ void wxTextEntry::SetMaxLength(unsigned long len)
     ::SendMessage(GetEditHwnd(), EM_LIMITTEXT, len, 0);
 }
 
+void wxTextEntry::ForceUpper()
+{
+    ConvertToUpperCase();
+
+    wxMSWWinStyleUpdater(GetEditHwnd()).TurnOn(ES_UPPERCASE);
+}
+
 // ----------------------------------------------------------------------------
 // hints
 // ----------------------------------------------------------------------------
@@ -946,7 +995,7 @@ void wxTextEntry::SetMaxLength(unsigned long len)
 
 bool wxTextEntry::SetHint(const wxString& hint)
 {
-    if ( wxGetWinVersion() >= wxWinVersion_Vista && wxUxThemeEngine::GetIfActive() )
+    if ( wxGetWinVersion() >= wxWinVersion_Vista && wxUxThemeIsActive() )
     {
         // notice that this message always works with Unicode strings
         //
@@ -963,7 +1012,7 @@ bool wxTextEntry::SetHint(const wxString& hint)
 
 wxString wxTextEntry::GetHint() const
 {
-    if ( wxUxThemeEngine::GetIfActive() )
+    if ( wxUxThemeIsActive() )
     {
         wchar_t buf[256];
         if ( ::SendMessage(GetEditHwnd(), EM_GETCUEBANNER,
@@ -983,7 +1032,6 @@ wxString wxTextEntry::GetHint() const
 
 bool wxTextEntry::DoSetMargins(const wxPoint& margins)
 {
-#if !defined(__WXWINCE__)
     bool res = true;
 
     if ( margins.x != -1 )
@@ -1002,22 +1050,113 @@ bool wxTextEntry::DoSetMargins(const wxPoint& margins)
     }
 
     return res;
-#else
-    return false;
-#endif
 }
 
 wxPoint wxTextEntry::DoGetMargins() const
 {
-#if !defined(__WXWINCE__)
     LRESULT lResult = ::SendMessage(GetEditHwnd(), EM_GETMARGINS,
                                     0, 0);
     int left = LOWORD(lResult);
     int top = -1;
     return wxPoint(left, top);
-#else
-    return wxPoint(-1, -1);
-#endif
+}
+
+// ----------------------------------------------------------------------------
+// input handling
+// ----------------------------------------------------------------------------
+
+bool wxTextEntry::ClickDefaultButtonIfPossible()
+{
+    return !wxIsAnyModifierDown() &&
+                wxWindow::MSWClickButtonIfPossible(
+                    wxWindow::MSWGetDefaultButtonFor(GetEditableWindow()));
+}
+
+// This function is also used by wxSpinCtrl, so make it extern to allow reusing
+// it from there.
+extern bool wxMSWTextEntryShouldPreProcessMessage(WXMSG* msg)
+{
+    // check for our special keys here: if we don't do it and the parent frame
+    // uses them as accelerators, they wouldn't work at all, so we disable
+    // usual preprocessing for them
+    if ( msg->message == WM_KEYDOWN )
+    {
+        const WPARAM vkey = msg->wParam;
+        if ( HIWORD(msg->lParam) & KF_ALTDOWN )
+        {
+            // Alt-Backspace is accelerator for "Undo"
+            if ( vkey == VK_BACK )
+                return false;
+        }
+        else // no Alt
+        {
+            // we want to process some Ctrl-foo and Shift-bar but no key
+            // combinations without either Ctrl or Shift nor with both of them
+            // pressed
+            const int ctrl = wxIsCtrlDown(),
+                      shift = wxIsShiftDown();
+            switch ( ctrl + shift )
+            {
+                default:
+                    wxFAIL_MSG( wxT("how many modifiers have we got?") );
+                    wxFALLTHROUGH;
+
+                case 0:
+                    switch ( vkey )
+                    {
+                        case VK_DELETE:
+                        case VK_HOME:
+                        case VK_END:
+                            return false;
+                    }
+                    break;
+
+                case 1:
+                    // either Ctrl or Shift pressed
+                    if ( ctrl )
+                    {
+                        switch ( vkey )
+                        {
+                            case 'A':
+                            case 'C':
+                            case 'V':
+                            case 'X':
+                            case VK_INSERT:
+                            case VK_DELETE:
+                            case VK_HOME:
+                            case VK_END:
+                            case VK_LEFT:
+                            case VK_RIGHT:
+                                return false;
+                        }
+                    }
+                    else // Shift is pressed
+                    {
+                        switch ( vkey )
+                        {
+                            case VK_INSERT:
+                            case VK_DELETE:
+                            case VK_HOME:
+                            case VK_END:
+                            case VK_LEFT:
+                            case VK_RIGHT:
+                                return false;
+                        }
+                    }
+                    break;
+
+                case 2:
+                    break;
+            }
+        }
+    }
+
+    return true;
+}
+
+bool wxTextEntry::MSWShouldPreProcessMessage(WXMSG* msg) const
+{
+    return wxMSWTextEntryShouldPreProcessMessage(msg);
 }
 
 #endif // wxUSE_TEXTCTRL || wxUSE_COMBOBOX
