@@ -6,7 +6,7 @@ intelligent Frame.  This one has a menu and a statusbar, is able to
 save and reload doodles, clear the workspace, and has a simple control
 panel for setting color and line thickness in addition to the popup
 menu that DoodleWindow provides.  There is also a nice About dialog
-implmented using an wx.html.HtmlWindow.
+implemented using an wx.html.HtmlWindow.
 """
 
 import sys
@@ -76,21 +76,17 @@ class DoodleFrame(wx.Frame):
         # size events using this sizer.
         self.SetSizer(box)
 
-
     def SaveFile(self):
         if self.filename:
             data = self.doodle.GetLinesData()
-            f = open(self.filename, 'wb')
-            pickle.dump(data, f)
-            f.close()
-
+            with open(self.filename, 'wb') as f:
+                pickle.dump(data, f)
 
     def ReadFile(self):
         if self.filename:
             try:
-                f = open(self.filename, 'rb')
-                data = pickle.load(f)
-                f.close()
+                with open(self.filename, 'rb') as f:
+                    data = pickle.load(f)
                 self.doodle.SetLinesData(data)
             except pickle.UnpicklingError:
                 wx.MessageBox("%s is not a doodle file." % self.filename,
@@ -242,7 +238,7 @@ class ControlPanel(wx.Panel):
             self.thknsBtns[x] = b
         self.thknsBtns[1].SetToggle(True)
 
-        # Make a colour indicator window, it is registerd as a listener
+        # Make a colour indicator window, it is registered as a listener
         # with the doodle window so it will be notified when the settings
         # change
         ci = ColourIndicator(self)
@@ -416,6 +412,17 @@ class DoodleApp(wx.App, InspectionMixin, SoftwareUpdate):
         self.SetAppDisplayName('SuperDoodle')
         return True
 
+    def MacReopenApp(self):
+        """
+        Restore the main frame (if it's minimized) when the Dock icon is
+        clicked on OSX.
+        """
+        top = self.GetTopWindow()
+        if top and top.IsIconized():
+            top.Iconize(False)
+        if top:
+            top.Raise()
+
 #----------------------------------------------------------------------
 
 if __name__ == '__main__':
